import re
class Subclass(object):

    def __init__(self, seed, length, lrange = None):
        self._seed   = seed
        if lrange is None or str(length) != '4':
            self._length = length
        else:
            if lrange == '00-04': self._length = 4
            else:                 self._length = 4.5
        self._loops  = []
        self.range   = lrange
        self._identifier = None

        self.dist_range  = None
        self.delta_range = None
        self.theta_range = None
        self.rho_range   = None

        self.dist  = None
        self.delta = None
        self.theta = None
        self.rho   = None

        self._ram_pat = None
        self._seq_pat = None
        self._exp_pat = None

        self.consensus = None

        self.subcode = 0
        self.subclassID = 'NULL'

    @property 
    def length(self): 
        return float(self._length)
        # if str(self._length) not in ['4A','4B']:
        #     return int(self._length)
        # else: return 4
    @property 
    def defining_length(self):
        return self._length
    @property 
    def size(self): return len(self._loops)
    @property 
    def loops(self): return self._loops
    @loops.setter
    def loops(self, loop): 
        self._loops.append(loop)
        self._loops[-1].order = self.length

    @property
    def identifier(self):
        if self._identifier is None:
            self._identifier = "_".join([y for y in sorted(["_".join([x.loopID,x.position]) for x in self.loops])])
        return self._identifier

    @property
    def ram_pat(self):
        return self._ram_pat
    @ram_pat.setter
    def ram_pat(self, value):
        self._ram_pat = self._motif(full = value)

    @property
    def seq_pat(self):
        return self._seq_pat
    @seq_pat.setter
    def seq_pat(self, value):
        self._seq_pat = self._motif(full = value)

    @property
    def exp_pat(self):
        return self._exp_pat
    @exp_pat.setter
    def exp_pat(self, value):
        self._exp_pat = self._motif(full = value)

    def add_topology(self, info, method):

        dist_code = {"A":'0-2', "B":'2-4', "C":'4-6', "D":'6-8', "E":'8-10', "F":'10-12', "G":'12-14', "H":'14-16', "I":'16-18',
                     "J":'18-20', "K":'20-22', "L":'22-24', "M":'24-26', "N":'26-28', "O":'28-30', "P":'30-32', "Q":'32-34', "R":'34-36', "S":'36-38',
                     "T":'38-40', "?":'40-42'}

        rho_code = {"A":'0-45', "B":'45-90', "C":'90-135', "D":'135-180', "E":'180-225', "F":'225-270', "G":'270-315', "H":'315-360', "?":'315-360'}

        #METHOD DEF
        if method == 'DS': #DENSITY SEARCH
            delta_code = {"A":'0-45', "B":'45-90', "C":'90-135', "D":'135-180', "E":'180-225', "F":'225-270', "G":'270-315', "H":'315-360', "?":'135-180',}
            theta_code = {"A":'0-45', "B":'45-90', "C":'90-135', "D":'135-180', "E":'180-225', "F":'225-270', "G":'270-315', "H":'315-360'}
        elif method == 'MCL': #MCL
            delta_code = {"A":'0-30', "B":'30-60', "C":'60-90', "D":'90-120', "E":'120-180', "F":'180-210', "G":'210-240', "H":'240-270',
                          "I":'270-300', "J":'300-330', "K":'330-360'}
            theta_code = {"A":'0-30', "B":'30-60', "C":'60-90', "D":'90-120', "E":'120-180', "F":'180-210', "G":'210-240', "H":'240-270',
                          "I":'270-300', "J":'300-330', "K":'330-360'}


        topology = info.split(':')[1].strip()

        self.dist_range  = dist_code[topology[0]]
        self.delta_range = delta_code[topology[1]]
        self.theta_range = theta_code[topology[2]]
        self.rho_range   = rho_code[topology[3]]

    def add_coordinates(self, info):

        data = info.split(':')[1].strip().split()
        self.dist  = data[2]
        self.delta = data[5]
        self.theta = data[8]
        self.rho   = data[11]

    def _motif(self, full):
        data = [re.sub(r',.\Z','',re.sub(r'[\(|\)]','',x.split('=')[-1])) for x in re.split('\)\(',full)]
        for d in range(len(data)):
            if re.search(r',',data[d]):
                data[d] = "[" + data[d] + "]"
        data = "-".join(data)

        return data

    def add_consensus(self, info, method, loops):
        INFO = info
        info = info.split(':')
        ini = None
        end = None
        if method == 'DS':
            l1 = len(info[0]) + 1 + len(info[1]) - len(info[1].lstrip())
            l2 = l1 + len(info[1].strip())
            z1 = loops[self._seed]
            the_first = True
            for loop in self.loops:
                loop.fix_align(l1 - loop.refer, l2 - loop.refer)
                if the_first:
                    # print "CON", ":".join(info)[loop.refer[0]-1:loop.refer[1]-1]
                    # print info[1]
                    # print INFO
                    # print INFO[loop.prel + z1 -2:loop.prel + z1 +2 + self.length]
                    # self.consensus = ":".join(info)[loop.refer[0]-1:loop.refer[1]-1]
                    # print consensus
                    # self.consensus = self.consensus[z1-2: z1 + 2 + self.length]
                    self.consensus = INFO[loop.prel + z1 -2:loop.prel + z1 +2 + int(self.length)]
                    the_first = False
                # if loop.id in loop_data:
                #     ini = loop.prel + loop.iniseq + loop_data[loop.id] - 2
                #     end = loop.prel + loop.iniseq + loop_data[loop.id] + loop.length + 2
                #     break
        if method == 'MCL':
            l1 = len(info[0]) + 1 + len(info[1]) - len(info[1].lstrip())
            l2 = l1 + len(info[1].strip())
            z0 = []
            the_first = True
            for loop in self.loops:
                z1 = loops[loop.loop_ID_pos()]
                loop.fix_align(l1 - loop.refer, l2 - loop.refer)
                if the_first:
                    z0 = [loop.prel + z1 -4, loop.prel + z1 +4 + int(self.length)]
                    the_first = False
                # else:
                #     if loop.prel + z1 -4 > z0[0]: z0[0] = loop.prel + z1 -4
                #     if loop.prel + z1 +4 + self.length < z0[1]: z0[1] = loop.prel + z1 +4 + self.length

            self.consensus = INFO[z0[0]:z0[1]]
                # the_first = False

    def toSQL(self, classcode, counter_subclass):
        subclassID = ".".join([classcode, str(counter_subclass)])
        command  = "INSERT INTO cluster_subclass (class_nid,subclass,name,size,seq_pat,exp_pat,ram_pat)"
        command += " VALUES (@classID,{2},'{1}',{0.size},'{0.seq_pat}','{0.exp_pat}','{0.ram_pat}');\n".format(self, subclassID, counter_subclass)
        command += "SET @subclassID = LAST_INSERT_ID();\n"
        loop_counter = 1
        for loop in self.loops:
            command += loop.toSQL(loop_counter)
            loop_counter += 1
        command += self.get_loops_params()
        return command


    def get_loops_params(self):
        command  = "SELECT CONCAT(FLOOR(MIN(ld.distance)),'-',CEIL(MAX(ld.distance))), "
        command += "CEIL(MAX(ld.distance)), FLOOR(MIN(ld.distance)),"
        command += "CONCAT(FLOOR(MIN(ld.theta)),'-',CEIL(MAX(ld.theta))), "
        command += "CEIL(MAX(ld.theta)), FLOOR(MIN(ld.theta)), "
        command += "CONCAT(FLOOR(MIN(ld.rho)),'-',CEIL(MAX(ld.rho))), "
        command += "CEIL(MAX(ld.rho)), FLOOR(MIN(ld.rho)),"
        command += "CONCAT(FLOOR(MIN(ld.delta)),'-',CEIL(MAX(ld.delta))), "
        command += "CEIL(MAX(ld.delta)),FLOOR(MIN(ld.delta)) "
        command += "FROM cluster_subclass cs, loop2cluster lc, loop_description ld WHERE "
        command += "cs.nid=@subclassID AND lc.cluster_nid=cs.nid AND ld.nid=lc.loop_nid "
        command += "INTO @Sr, @Sx, @Sm, @Tr, @Tx, @Tm, @Rr, @Rx, @Rm, @Dr, @Dx, @Dm;\n"
        command += "UPDATE cluster_subclass SET dist_range=@Sr, theta_range=@Tr, rho_range=@Rr, delta_range=@Dr, "
        command += "dist_range_min=@Sm, theta_range_min=@Tm, rho_range_min=@Rm, delta_range_min=@Dm, "
        command += "dist_range_max=@Sx, theta_range_max=@Tx, rho_range_max=@Rx, delta_range_max=@Dx "
        command += "WHERE nid=@subclassID;\n"
        return command

    def __cmp__(self, other):
        return self.size.__cmp__(other.size)
        

    # def add_consensus(self, info, loop_data, method):
    #     ini = None
    #     end = None
    #     if method == 'DS':
    #         for loop in self.loops:
    #             if loop.id in loop_data:
    #                 ini = loop.prel + loop.iniseq + loop_data[loop.id] - 2
    #                 end = loop.prel + loop.iniseq + loop_data[loop.id] + loop.length + 2
    #                 break
    #     elif method == 'MCL':
    #         for loop in self.loops:
    #             if loop.id in loop_data:
    #                 new_ini = loop.prel + loop.iniseq + loop_data[loop.id]
    #                 new_end = loop.prel + loop.iniseq + loop_data[loop.id] + loop.length
    #                 if ini is None:
    #                     ini = new_ini
    #                     end = new_end
    #                 else:
    #                     if new_ini < end:
    #                         if new_ini > ini: ini = new_ini
    #                         if new_end < end: end = new_end
    #         if ini is not None:
    #             ini = ini - 4
    #             end = end + 4
    #         else:
    #             ini = 0
    #             end = -1
    #     self.consensus = info[ini:end]
class Consensus(object):
    def __init__(self, length, consensus, gtype, subclasses, method):
        self.consensus = consensus
        self.length = length
        self.gtype = gtype
        self.subclasses = subclasses
        self.method = method
        self._size = -1
    @property
    def size(self):
        if self._size == -1:
            self._size = 0
            for s in self.subclasses:
                self._size += s.size
        return self._size

    def __cmp__(self, other):
        return self.size.__cmp__(other.size)

    def toSQL(self, classcode):
        length = self.length
        if self.method == 'MCL' and length == 4: length = '4S'
        elif self.method == 'MCL' and length == 4.5: length = '4M'
        else: length = int(length)
        # length  = self.length if self.length not in ['4A','4B'] else '4S' if self.length == '4A' else '4M'
        classID = ".".join([self.gtype,str(length),str(classcode)]) 
        command  = "INSERT INTO cluster_class (method,type,length,class,name,size,consensus) "
        command += "VALUES (@databasemethod,'{0.gtype}','{3}',{1},'{2}',{0.size},'{0.consensus}');\n".format(self, classcode, classID, length)
        command += "SET @classID = LAST_INSERT_ID();\n"
        counter_subclass = 1
        for s in self.subclasses:
            command += s.toSQL(classID,counter_subclass)
            counter_subclass += 1
        return command

class Cclass(object):
    def __init__(self, kltype):
        self._kltype       = kltype
        self._subclasses   = {}
        self._lastsubclass = None

    @property 
    def subclasses(self): return self._subclasses
    @subclasses.setter
    def subclasses(self, value): 
        self._subclasses.setdefault(value.length,[]).append(value)
        self._lastsubclass = value

    @property 
    def lastsubclass(self): return self._lastsubclass

    def toSQL(self, method):
        info = {}
        consdata = {}
        donesubclass = set()
        command = "SELECT nid FROM method WHERE name='{0}' INTO @databasemethod;\n".format(method)
        for l in self.subclasses:
            info.setdefault(l,{})
            for s in self.subclasses[l]:
                if method == 'MCL' and s.identifier in donesubclass: continue
                if s.size >= 3:
                    info[l].setdefault(s.consensus,[]).append(s)
                    if method == 'MCL': donesubclass.add(s.identifier)
            for c in info[l]:
                consdata.setdefault(l,[]).append(Consensus(l,c,self._kltype, sorted(info[l][c], reverse=True), method))
                # info[l][c] = sorted(info[l][c], reverse=True)


        for l in sorted(consdata):
            class_count = 1
            # print 'LENGTH ', l
            consdata[l] = sorted(consdata[l], reverse=True)
            for c in consdata[l]:
                # if method == 'MCL' and c.identifier in donesubclass: continue
                command += c.toSQL(class_count)
                class_count += 1
                # if method == 'MCL': donesubclass.add(c.identifier)
                # subclass_count = 0
                # print '\tCONS ', c.consensus, 'SIZE ', c.size
                # for s in c.subclasses:
                #     print '\t\tSIZE ', s.size, ' SEED ', s._seed
       
        return command

    # def toSQL(self):
    #     for cla in sorted(self.subclasses):
    #         print 'LENGTH: ', cla
    #         for sub in self.subclasses[cla]:
    #             print 'SEED: ', sub._seed, ' SIZE: ', sub.size, ' S: ', len(sub.seq_pat.split('-')), ' R: ', len(sub.ram_pat.split('-')), ' E: ', len(sub.exp_pat.split('-'))
    #             print 'CONS: ', sub.consensus
    #             print sub.toSQL()
    #             for loop in sub.loops:
    #                 print loop.toSQL()

class Loop(object):
    def __init__(self, info):
        self.loopID   = info.split(':')[0].strip().split()[0]
        self.position = info.split(':')[0].strip().split()[3]
        self.length   = int(info.split(':')[0].strip().split()[4])
        self.seq      = info.split(':')[1].split('|')[0]
        self.exp      = None
        self.ram      = None
        self.scs      = None

        self.score    = float(info.split(':')[1].split('|')[1].strip())
        self.prel     = None
        self.order    = None
        self.refer    = None

    def loop_ID_pos(self): return tuple([self.loopID,self.position])
    def toSQL(self, order):
        return self.loop_nid_SQL() + self.loop2cluster(order)

    def loop_nid_SQL(self):
        command  = "SELECT lc.loop_id INTO @loopNID FROM loop2chain lc, chain c\n"
        command += " WHERE c.pdb='{0[0]}' and c.chain='{0[1]}'\n".format(self.loopID.split('_'))
        command += "  AND lc.chain=c.nid\n"
        command += "  AND lc.position={0};\n".format(self.position)
        return command

    def loop2cluster(self, order):
        command  = "INSERT INTO loop2cluster (cluster_nid,loop_nid,clust_order,score,seq_ali,ss_ali,exp_ali,ram_ali)\n"
        command += " VALUES (@subclassID,@loopNID,{1},{0.score:.3f},'{0.seq}','{0.scs}','{0.exp}','{0.ram}');\n".format(self, order)
        return command

    def fix_align(self, ini, end):
        self.seq = re.sub(' ','-',self.seq[int(ini):int(end)])
        self.exp = re.sub(' ','-',self.exp[int(ini):int(end)])
        self.ram = re.sub(' ','-',self.ram[int(ini):int(end)])
        self.scs = re.sub(' ','-',self.scs[int(ini):int(end)])

    # @property
    # def iniseq(self):
    #     return len(self.seq) - len(self.seq.lstrip())

    # @property
    # def endseq(self):
    #     return -(len(self.seq) - len(self.seq.rstrip()))

    def add_surface(self, info):

        self.exp   = info.split(':')[1].split('|')[0]
        self.refer = len(info.split(':')[0]) + 1 
        position   = re.search('\[\s*\d+\s*\+\s*(\-*\d+),\s*\d+\s*\+\s*(\-*\d+)\]', info.split(':')[1].split('|')[-1])
        p1,p2      = int(position.group(1)), int(position.group(2))
        # correct    = self.prel + 1 + len(self.exp) - len(self.exp.lstrip())
        self.prel = len(info.split(':')[0]) + 1 + len(info.split(':')[1].split('|')[0]) - len(info.split(':')[1].split('|')[0].lstrip())
        # self.refer.extend([p1+correct, p2+correct])

    def add_ramachandran(self, info):
        self.ram   = info.split(':')[1].split('|')[0]

    def add_secondary_str(self, info):
        self.scs   = info.split(':')[1].split('|')[0]

    

    # def fix_align(self, ini, end):
    #     self.seq = re.sub(' ','-',self.seq[int(ini):int(end)])
    #     self.exp = re.sub(' ','-',self.exp[int(ini):int(end)])
    #     self.ram = re.sub(' ','-',self.ram[int(ini):int(end)])
    #     self.scs = re.sub(' ','-',self.scs[int(ini):int(end)])

    # def __repr__(self):
    #     data = []

    #     data.append("SELECT nid FROM loop_description WHERE loop_id='{0.id}' INTO @loopNID;".format(self))
    #     data.append("INSERT INTO loop2cluster (cluster_nid,loop_nid,clust_order,score,seq_ali,ss_ali,exp_ali,ram_ali)" + \
    #                 " VALUES (LAST_INSERT_ID(),@loopNID,{0.order},{0.score:.3f},'{0.seq}','{0.scs}','{0.exp}','{0.ram}');".format(self))

    #     return "\n".join(data)


__all__ = [
    "Singleton",
    "Butler",
    "File",
    "FileError",
    "StorableObject",
    "Executable",
    "Path",
    "IndexedNum",
    "JSONer"
]

from .singleton      import Singleton
from .butler         import Butler
from .file           import (File, FileError)
from .StorableObject import StorableObject
from .Executable     import Executable
from .Path           import Path
from .IndexedNum     import IndexedNum
from .JSONer         import JSONer



import os, sys, random, math
import numpy
from BioLib.Tools.BioExceptions import ResidueTriadError

class SplitPotentialsData(object):
    '''
        Contain the SplitPotential Calculation
    '''
    def __init__(self, ori_ppResidues, ori_combined, ori_ppRE, ori_localppRE, ori_ppEnvironment, ori_ppDist,
                 cof_ppResidues, cof_combined, cof_ppRE, cof_localppRE, cof_ppEnvironment, cof_ppDist,
                 max_ppResidues, max_ppEnvironment, max_ppRE, max_combined,
                 min_ppResidues, min_ppEnvironment, min_ppRE, min_combined,
                 men_ppResidues, men_ppEnvironment, men_ppRE, men_combined):

        self.original    = {"D-Score":ori_ppResidues, "D-Energy":ori_combined, "D-Eaa3Denv":ori_ppRE,
                            "D-Elocal":ori_localppRE, "D-E3Denv":ori_ppEnvironment, "D-E3D":ori_ppDist,
                            "C-Score":cof_ppResidues, "C-Energy":cof_combined, "C-Eaa3Denv":cof_ppRE,
                            "C-Elocal":cof_localppRE, "C-E3Denv":cof_ppEnvironment, "C-E3D":cof_ppDist,
                            "M-Score":men_ppResidues, "M-Energy":men_combined, "M-Eaa3Denv":men_ppRE,
                                                      "M-E3Denv":men_ppEnvironment, 
                            "U-Score":max_ppResidues, "U-Energy":max_combined, "U-Eaa3Denv":max_ppRE,
                                                      "U-E3Denv":max_ppEnvironment, 
                            "L-Score":min_ppResidues, "L-Energy":min_combined, "L-Eaa3Denv":min_ppRE,
                                                      "L-E3Denv":min_ppEnvironment}
        
        self.normalized = {"D-ZScore":0,"D-ZEnergy":0,"D-ZEaa3Denv":0,"D-ZElocal":0,"D-ZE3Denv":0,
                           "C-ZScore":0,"C-ZEnergy":0,"C-ZEaa3Denv":0,"C-ZElocal":0,"C-ZE3Denv":0,
                           "M-ZScore":0,"M-ZEnergy":0,"M-ZEaa3Denv":0,              "M-ZE3Denv":0,
                           "U-ZScore":0,"U-ZEnergy":0,"U-ZEaa3Denv":0,              "U-ZE3Denv":0,
                           "L-ZScore":0,"L-ZEnergy":0,"L-ZEaa3Denv":0,              "L-ZE3Denv":0}

        self.original_key    = ["D-Score", "D-Energy", "D-Eaa3Denv", "D-Elocal", "D-E3Denv", "D-E3D",
                                "C-Score", "C-Energy", "C-Eaa3Denv", "C-Elocal", "C-E3Denv", "C-E3D",
                                "M-Score", "M-Energy", "M-Eaa3Denv",             "M-E3Denv",
                                "U-Score", "U-Energy", "U-Eaa3Denv",             "U-E3Denv",
                                "L-Score", "L-Energy", "L-Eaa3Denv",             "L-E3Denv"]
        self.normalized_key  = ["D-ZScore","D-ZEnergy","D-ZEaa3Denv","D-ZElocal","D-ZE3Denv",
                                "C-ZScore","C-ZEnergy","C-ZEaa3Denv","C-ZElocal","C-ZE3Denv",
                                "M-ZScore","M-ZEnergy","M-ZEaa3Denv",            "M-ZE3Denv",
                                "U-ZScore","U-ZEnergy","U-ZEaa3Denv",            "U-ZE3Denv",
                                "L-ZScore","L-ZEnergy","L-ZEaa3Denv",            "L-ZE3Denv"] 
    """
        Getters
    """
    def get_value_by_label(self, label):
        if self.original.has_key(label):
            return self.original[label]
        
    def get_value_by_label_str(self,label):
        return "%.3e" %self.get_value_by_label(label = label)
    
    def get_tabbed_labels(self):
        return "\t".join(self.original_key)
    
    def get_tabbed_values(self):
        outarray = []
        for label in self.original_key:
            outarray.append(self.get_value_by_label_str(label = label))
        return "\t".join(outarray)
    
    def get_normalized_value_by_label(self, label):
        if self.normalized.has_key(label):
            return self.normalized[label]
        
    def get_normalized_value_by_label_str(self,label):
        return "%.3e" %self.get_normalized_value_by_label(label = label)
    
    def get_normalized_tabbed_labels(self):
        return "\t".join(self.normalized_key)
    
    def get_normalized_tabbed_values(self):
        outarray = []
        for label in self.normalized_key:
            outarray.append(self.get_normalized_value_by_label_str(label = label))
        return "\t".join(outarray)
    
    def get_all_tabbed_labels(self):
        return "\t".join((self.get_tabbed_labels(),self.get_normalized_tabbed_labels()))
    
    def get_all_tabbed_values(self):
        return "\t".join((self.get_tabbed_values(),self.get_normalized_tabbed_values()))
    
    """
        Setters
    """
    def set_random_vector(self, vector, label):    

        if not self.normalized.has_key(label):
            return
        
        mean = numpy.mean(vector)
        std = numpy.std(vector)
        
        original_label = label[:2]+label[3:]
        
        self.normalized[label] = (self.original[original_label] - mean) / std

    """
        toString
    """
    def toString(self, header = True, ID = None, headertag = None):
        
        line = []
        
        if ID == None:    
            if header:
                line.append(self.get_all_tabbed_labels())
            line.append(self.get_all_tabbed_values())
        else:
            if header:
                line.append(str(headertag) + "\t" + self.get_all_tabbed_labels())
            line.append(str(ID) + "\t" + self.get_all_tabbed_values()) 
        
        return "\n".join(line)
    

class SplitPotentialsMatrix(object):
    
    """
        Constructor
    """
    def __init__(self, sp_matrix = 'ppi3_cbR12.out', cutoff = 12):
        
        sp_file = os.path.abspath(os.path.join(os.path.dirname(__file__), sp_matrix))
        if not os.path.isfile(sp_file):
            sys.stderr.write("Can not read the potential file %s\n\n" % sp_file)
            sys.exit()
            
        self.file = sp_file    
        self.cutoff = cutoff
        
        self.ppResidues     = {}        # Pair Potentials of Residues
        self.ppEnvironment  = {}        # Pair Potentials of Environment
        self.localppRE      = {}        # Local Pair Potentials of Residues and Environment
        self.ppRE           = {}        # Pair Potentials of Residues and Environment
        self.ppDist         = {}        # Standard Pair Potentials of Distances
#       self.envWeights     = {}        # Solution for Environmental Weights
#       self.ppAccumPair    = {}        # Pair Potentials of Accumulated Pairs
        
        self.realInteraction = None
        
#       self.stats_ppResidues = {}      # Mean, Max and Min for each pair of Pair Potentials of Residues
#       self.stats_ppEnvironment = {}   # Mean, Max and Min for each pair of Pair Potentials of Environment
#       self.stats_ppRE = {}            # Mean, Max and Min for each pair of Pair Potentials of Residues and Environment
        
        tag = 0
        pp_fd = open(self.file)
        for line in pp_fd:
            if line.startswith("PAIR-POTENTIALS OF RESIDUES AND ENVIRONMENTS"):             tag = 4
            elif line.startswith("PAIR-POTENTIALS OF ENVIRONMENTS"):                        tag = 2
            elif line.startswith("LOCAL PAIR-POTENTIALS OF RESIDUES AND ENVIRONMENTS"):     tag = 3
            elif line.startswith("PAIR-POTENTIALS OF RESIDUES"):                            tag = 1
            elif line.startswith("STANDARD PAIR-POTENTIALS OF DISTANCES"):                  tag = 5
#           elif line.startswith("SOLUTION FOR ENVIRONMENTAL WEIGHTS"):                     tag = 6
#           elif line.startswith("PAIR-POTENTIALS OF ACCUMULATED PAIRS"):                   tag = 7
            elif line.startswith("END"):                                                    tag = 0
            
            self.process_file_line(line = line, tag = tag)

        pp_fd.close()

    def add_interaction(self, interaction):
        self.realInteraction = interaction
        real_interacting_residues = interaction.get_interacting_residues(c_type = 'CB', max_distance = 13, uniq=False)
        self.str1_contact_pos = [x.get_num() for x in real_interacting_residues[0]]
        self.str2_contact_pos = [x.get_num() for x in real_interacting_residues[1]]
        self.contact_distances = real_interacting_residues[2]
        
    def calculateSplitPotentials(self, randoms = 100):

        # Calculate Original Split Potentials
        ori_ppResidues,     cof_ppResidues,     max_ppResidues,     min_ppResidues,        men_ppResidues      = 0, 0, 0, 0 ,0
        ori_ppEnvironment,  cof_ppEnvironment,  max_ppEnvironment,  min_ppEnvironment,     men_ppEnvironment   = 0, 0, 0, 0 ,0
        ori_localppRE,      cof_localppRE       = 0, 0
        ori_ppRE,           cof_ppRE,           max_ppRE,           min_ppRE,               men_ppRE           = 0, 0, 0, 0 ,0
        ori_ppDist,         cof_ppDist          = 0, 0
        ori_combined,       cof_combined,       max_combined,       min_combined,           men_combined        = 0, 0, 0, 0 ,0

        for x in xrange(len(self.str1_contact_pos)):
            residue1 = self.realInteraction.get_structure1().get_residue_by_num(self.str1_contact_pos[x])
            residue2 = self.realInteraction.get_structure2().get_residue_by_num(self.str2_contact_pos[x])
            
            try:
                triad1 = residue1.get_triad()
                triad2 = residue2.get_triad()
            except ResidueTriadError as e:
                sys.stdout.write(str(e))
                sys.stdout.flush()
                continue
            
            positional_ppResidues   = float(self.get_ppResidues(k1=triad1[0:1],k2=triad2[0:1], distance=self.contact_distances[x]))
            ori_ppResidues          += positional_ppResidues
            maximum_ppResidues      = float(self.get_ppResidues_stat(k1=triad1[0:1],k2=triad2[0:1], stat='upper'))
            max_ppResidues          += maximum_ppResidues
            minimum_ppResidues      = float(self.get_ppResidues_stat(k1=triad1[0:1],k2=triad2[0:1], stat='lower'))
            min_ppResidues          += minimum_ppResidues
            mean_ppResidues         = float(self.get_ppResidues_stat(k1=triad1[0:1],k2=triad2[0:1], stat='mean'))
            men_ppResidues          += mean_ppResidues
                    
            positional_ppEnvironment = float(self.get_ppEnvironment(k1 = triad1[2:],k2 = triad2[2:], distance=self.contact_distances[x]))
            ori_ppEnvironment       += positional_ppEnvironment
            maximum_ppEnvironment   = float(self.get_ppEnvironment_stat(k1=triad1[2:],k2=triad2[2:], stat='upper'))
            max_ppEnvironment       += maximum_ppEnvironment
            minimum_ppEnvironment   = float(self.get_ppEnvironment_stat(k1=triad1[2:],k2=triad2[2:], stat='lower'))
            min_ppEnvironment       += minimum_ppEnvironment
            mean_ppEnvironment      = float(self.get_ppEnvironment_stat(k1=triad1[2:],k2=triad2[2:], stat='mean'))
            men_ppEnvironment       += mean_ppEnvironment
            
            positional_localppRE    = float(self.get_localppRE(k=triad1)) + float(self.get_localppRE(k=triad2))
            ori_localppRE           += positional_localppRE
            
            positional_ppRE         = float(self.get_ppRE(k1 = triad1, k2 = triad2, distance=self.contact_distances[x]))
            ori_ppRE                += positional_ppRE
            maximum_ppRE            = float(self.get_ppRE_stat(k1=triad1,k2=triad2, stat='upper'))
            max_ppRE                += maximum_ppRE
            minimum_ppRE            = float(self.get_ppRE_stat(k1=triad1,k2=triad2, stat='lower'))
            min_ppRE                += minimum_ppRE
            mean_ppRE               = float(self.get_ppRE_stat(k1=triad1,k2=triad2, stat='mean'))
            men_ppRE                += mean_ppRE
            
            positional_ppDist       = float(self.get_ppDist(k=self.contact_distances[x]))
            ori_ppDist              += positional_ppDist

            positional_combined     = positional_ppRE + positional_ppDist - positional_localppRE - positional_ppEnvironment
            ori_combined            += positional_combined
            maximum_combined        = maximum_ppRE + positional_ppDist - positional_localppRE - maximum_ppEnvironment
            max_combined            += maximum_combined
            minimum_combined        = minimum_ppRE + positional_ppDist - positional_localppRE - minimum_ppEnvironment
            min_combined            += minimum_combined
            mean_combined           = mean_ppRE + positional_ppDist - positional_localppRE - mean_ppEnvironment
            men_combined            += mean_combined
            
            if math.floor(self.contact_distances[x]) <= self.cutoff:
                cutoff_ppResidues   = float(self.get_ppResidues(k1=triad1[0:1],k2=triad2[0:1], distance=self.cutoff))
                cof_ppResidues      += cutoff_ppResidues
                
                cutoff_ppEnvironment = float(self.get_ppEnvironment(k1 = triad1[2:],k2 = triad2[2:], distance=self.cutoff))
                cof_ppEnvironment   += cutoff_ppEnvironment
                
                cutoff_localppRE    = float(self.get_localppRE(k=triad1)) + float(self.get_localppRE(k=triad2))
                cof_localppRE       += cutoff_localppRE
                
                cutoff_ppRE         = float(self.get_ppRE(k1 = triad1, k2 = triad2, distance=self.cutoff))
                cof_ppRE            += cutoff_ppRE
                
                cutoff_ppDist       = float(self.get_ppDist(k=self.cutoff))
                cof_ppDist          += cutoff_ppDist
    
                cutoff_combined     = cutoff_ppRE + cutoff_ppDist - cutoff_localppRE - cutoff_ppEnvironment
                cof_combined        += cutoff_combined
     
        result = SplitPotentialsData(ori_ppResidues = ori_ppResidues, ori_combined = ori_combined, ori_ppRE = ori_ppRE, 
                                     ori_localppRE = ori_localppRE, ori_ppEnvironment = ori_ppEnvironment, ori_ppDist = ori_ppDist,
                                     cof_ppResidues = cof_ppResidues, cof_combined = cof_combined, cof_ppRE = cof_ppRE,
                                     cof_localppRE = cof_localppRE, cof_ppEnvironment = cof_ppEnvironment, cof_ppDist = cof_ppDist,
                                     max_ppResidues = max_ppResidues, max_ppEnvironment = max_ppEnvironment, max_ppRE = max_ppRE, max_combined = max_combined,
                                     min_ppResidues = min_ppResidues, min_ppEnvironment = min_ppEnvironment, min_ppRE = min_ppRE, min_combined = min_combined,
                                     men_ppResidues = men_ppResidues, men_ppEnvironment = men_ppEnvironment, men_ppRE = men_ppRE, men_combined = men_combined)
        
        # If we are not going to do randoms we can return now
        if randoms == 0:
            #return (ori_ppResidues, ori_combined, ori_ppRE, ori_localppRE, ori_ppEnvironment, ori_ppDist)
            return result
        
        # Calculate randoms: Generate zero positions
        rad_ppResidues      = numpy.zeros((randoms+1), float)
        rdC_ppResidues      = numpy.zeros((randoms+1), float)
        rdM_ppResidues      = numpy.zeros((randoms+1), float)
        rdU_ppResidues      = numpy.zeros((randoms+1), float)
        rdL_ppResidues      = numpy.zeros((randoms+1), float)
        rad_ppEnvironment   = numpy.zeros((randoms+1), float)
        rdC_ppEnvironment   = numpy.zeros((randoms+1), float)
        rdM_ppEnvironment   = numpy.zeros((randoms+1), float)
        rdU_ppEnvironment   = numpy.zeros((randoms+1), float)
        rdL_ppEnvironment   = numpy.zeros((randoms+1), float)
        rad_localppRE       = numpy.zeros((randoms+1), float)
        rdC_localppRE       = numpy.zeros((randoms+1), float)
        rad_ppRE            = numpy.zeros((randoms+1), float)
        rdC_ppRE            = numpy.zeros((randoms+1), float)
        rdM_ppRE            = numpy.zeros((randoms+1), float)
        rdU_ppRE            = numpy.zeros((randoms+1), float)
        rdL_ppRE            = numpy.zeros((randoms+1), float)
        rad_ppDist          = numpy.zeros((randoms+1), float)
        rdC_ppDist          = numpy.zeros((randoms+1), float)
        rad_combined        = numpy.zeros((randoms+1), float)
        rdC_combined        = numpy.zeros((randoms+1), float)
        rdM_combined        = numpy.zeros((randoms+1), float)
        rdU_combined        = numpy.zeros((randoms+1), float)
        rdL_combined        = numpy.zeros((randoms+1), float)
        
        for rad in xrange(randoms):
            # Create random selection of positions for str1 and str2
            # We pick positions by array position as we do not care what are we picking and will avoid
            #    errors in case of gaps in the structure.
            # We need to take into account that we can actually have more interactions than Aa in the 
            #    protein, as the same Aa can have several interactions
            rad_str1 = self.randomise_interaface(list=self.str1_contact_pos, 
                                                 range=len(self.realInteraction.get_structure1().get_residues()))
            rad_str2 = self.randomise_interaface(list=self.str2_contact_pos, 
                                                 range=len(self.realInteraction.get_structure2().get_residues()))
            for x in xrange(len(rad_str1)):
                residue1 = self.realInteraction.get_structure1().get_residues()[rad_str1[x]]
                residue2 = self.realInteraction.get_structure2().get_residues()[rad_str2[x]]
                
                # We keep in the triad the exposition and secondary structure
                try:
                    triad1 = residue1.get_triad()[:4] + self.realInteraction.get_structure1().get_residue_by_num(self.str1_contact_pos[x]).get_triad()[4:]
                    triad2 = residue2.get_triad()[:4] + self.realInteraction.get_structure2().get_residue_by_num(self.str2_contact_pos[x]).get_triad()[4:]
                except ResidueTriadError as e:
                    sys.stdout.write(str(e))
                    sys.stdout.flush()
                    continue

                rad_ppResidues[rad]     += float(self.get_ppResidues(k1=triad1[0:1],k2=triad2[0:1],distance=self.contact_distances[x]))
                rdM_ppResidues[rad]     += float(self.get_ppResidues_stat(k1=triad1[0:1],k2=triad2[0:1], stat='mean'))
                rdU_ppResidues[rad]     += float(self.get_ppResidues_stat(k1=triad1[0:1],k2=triad2[0:1], stat='upper'))
                rdL_ppResidues[rad]     += float(self.get_ppResidues_stat(k1=triad1[0:1],k2=triad2[0:1], stat='lower'))
                
                rad_ppEnvironment[rad]  += float(self.get_ppEnvironment(k1 = triad1[2:],k2 = triad2[2:],distance=self.contact_distances[x]))
                rdM_ppEnvironment[rad]  += float(self.get_ppEnvironment_stat(k1=triad1[2:],k2=triad2[2:], stat='mean'))
                rdU_ppEnvironment[rad]  += float(self.get_ppEnvironment_stat(k1=triad1[2:],k2=triad2[2:], stat='upper'))
                rdL_ppEnvironment[rad]  += float(self.get_ppEnvironment_stat(k1=triad1[2:],k2=triad2[2:], stat='lower'))
                
                rad_localppRE[rad]      += float(self.get_localppRE(k=triad1)) + float(self.get_localppRE(k=triad2))
                
                rad_ppRE[rad]           += float(self.get_ppRE(k1 = triad1, k2 = triad2,distance=self.contact_distances[x]))
                rdM_ppRE[rad]           += float(self.get_ppRE_stat(k1=triad1,k2=triad2, stat='mean'))
                rdU_ppRE[rad]           += float(self.get_ppRE_stat(k1=triad1,k2=triad2, stat='upper'))
                rdL_ppRE[rad]           += float(self.get_ppRE_stat(k1=triad1,k2=triad2, stat='lower'))
                
                rad_ppDist[rad]         += float(self.get_ppDist(k=self.contact_distances[x]))
                
                
                if math.floor(self.contact_distances[x]) <= self.cutoff:
                    rdC_ppResidues[rad]     += float(self.get_ppResidues(k1=triad1[0:1],k2=triad2[0:1], distance=self.cutoff))
                
                    rdC_ppEnvironment[rad]  += float(self.get_ppEnvironment(k1 = triad1[2:],k2 = triad2[2:], distance=self.cutoff))
                
                    rdC_localppRE[rad]      += float(self.get_localppRE(k=triad1)) + float(self.get_localppRE(k=triad2))
                
                    rdC_ppRE[rad]           += float(self.get_ppRE(k1 = triad1, k2 = triad2, distance=self.cutoff))
                
                    rdC_ppDist[rad]         += float(self.get_ppDist(k=self.cutoff))
                    
 
            rad_combined[rad] = rad_ppRE[rad] + rad_ppDist[rad] - rad_localppRE[rad] - rad_ppEnvironment[rad]
            rdC_combined[rad] = rdC_ppRE[rad] + rdC_ppDist[rad] - rdC_localppRE[rad] - rdC_ppEnvironment[rad]
            rdM_combined[rad] = rdM_ppRE[rad] + rad_ppDist[rad] - rad_localppRE[rad] - rdM_ppEnvironment[rad]
            rdU_combined[rad] = rdU_ppRE[rad] + rad_ppDist[rad] - rad_localppRE[rad] - rdU_ppEnvironment[rad]
            rdL_combined[rad] = rdL_ppRE[rad] + rad_ppDist[rad] - rad_localppRE[rad] - rdL_ppEnvironment[rad]

        #sys.stderr.write("\n")

        # Add the original as an extra random
        rad_ppResidues[-1]      = ori_ppResidues
        rad_ppEnvironment[-1]   = ori_ppEnvironment
        rad_localppRE[-1]       = ori_localppRE
        rad_ppRE[-1]            = ori_ppRE
        rad_ppDist[-1]          = ori_ppDist
        rad_combined[-1]        = ori_combined
        
        
        result.set_random_vector(vector = rad_ppResidues,     label="D-ZScore")
        result.set_random_vector(vector = rad_ppEnvironment,  label="D-ZE3Denv")
        result.set_random_vector(vector = rad_localppRE,      label="D-ZElocal")
        result.set_random_vector(vector = rad_ppRE,           label="D-ZEaa3Denv")
        result.set_random_vector(vector = rad_combined,       label="D-ZEnergy")
        
        result.set_random_vector(vector = rdC_ppResidues,     label="C-ZScore")
        result.set_random_vector(vector = rdC_ppEnvironment,  label="C-ZE3Denv")
        result.set_random_vector(vector = rdC_localppRE,      label="C-ZElocal")
        result.set_random_vector(vector = rdC_ppRE,           label="C-ZEaa3Denv")
        result.set_random_vector(vector = rdC_combined,       label="C-ZEnergy")
        
        result.set_random_vector(vector = rdM_ppResidues,     label="M-ZScore")
        result.set_random_vector(vector = rdM_ppEnvironment,  label="M-ZE3Denv")
        result.set_random_vector(vector = rdM_ppRE,           label="M-ZEaa3Denv")
        result.set_random_vector(vector = rdM_combined,       label="M-ZEnergy")
        
        result.set_random_vector(vector = rdU_ppResidues,     label="U-ZScore")
        result.set_random_vector(vector = rdU_ppEnvironment,  label="U-ZE3Denv")
        result.set_random_vector(vector = rdU_ppRE,           label="U-ZEaa3Denv")
        result.set_random_vector(vector = rdU_combined,       label="U-ZEnergy")
        
        result.set_random_vector(vector = rdL_ppResidues,     label="L-ZScore")
        result.set_random_vector(vector = rdL_ppEnvironment,  label="L-ZE3Denv")
        result.set_random_vector(vector = rdL_ppRE,           label="L-ZEaa3Denv")
        result.set_random_vector(vector = rdL_combined,       label="L-ZEnergy")
        

        return result

    """
        Getters
    """
    def get_ppResidues(self,k1,k2,distance):
        d = int(math.floor(distance))
        if self.ppResidues.has_key(k1) and self.ppResidues[k1].has_key(k2):
            return self.ppResidues[k1][k2][d]
        elif self.ppResidues.has_key(k2) and self.ppResidues[k2].has_key(k1):
            return self.ppResidues[k2][k1][d]
        else:
            return 0
        
    def get_ppResidues_stat(self,k1,k2,stat):
        data = None
        if self.ppResidues.has_key(k1) and self.ppResidues[k1].has_key(k2):
            data = numpy.array(self.ppResidues[k1][k2], dtype='float64')
        elif self.ppResidues.has_key(k2) and self.ppResidues[k2].has_key(k1):
            data = numpy.array(self.ppResidues[k2][k1], dtype='float64')
        else:
            return 0
        
        if stat == 'upper':
            return numpy.max(data)
        elif stat == 'lower':
            return numpy.min(data)
        elif stat == 'mean':
            return numpy.mean(data)
        
    def get_ppEnvironment(self,k1,k2,distance):
        d = int(math.floor(distance))
        if self.ppEnvironment.has_key(k1) and self.ppEnvironment[k1].has_key(k2):
            return self.ppEnvironment[k1][k2][d]
        elif self.ppEnvironment.has_key(k2) and self.ppEnvironment[k2].has_key(k1):
            return self.ppEnvironment[k2][k1][d]
        else:
            return 0

    def get_ppEnvironment_stat(self,k1,k2,stat):
        data = None
        if self.ppEnvironment.has_key(k1) and self.ppEnvironment[k1].has_key(k2):
            data = numpy.array(self.ppEnvironment[k1][k2], dtype='float64')
        elif self.ppEnvironment.has_key(k2) and self.ppEnvironment[k2].has_key(k1):
            data = numpy.array(self.ppEnvironment[k2][k1], dtype='float64')
        else:
            return 0
        
        # UPPER and LOWER are inverted in this potential
        if stat == 'upper':
            return numpy.min(data)
        elif stat == 'lower':
            return numpy.max(data)
        elif stat == 'mean':
            return numpy.mean(data)  
              
    def get_localppRE(self,k):
        return self.localppRE[k]
    
    def get_ppRE(self,k1,k2,distance):
        d = int(math.floor(distance))
        if self.ppRE.has_key(k1) and self.ppRE[k1].has_key(k2):
            return self.ppRE[k1][k2][d]
        elif self.ppRE.has_key(k2) and self.ppRE[k2].has_key(k1):
            return self.ppRE[k2][k1][d]
        else:
            return 0

    def get_ppRE_stat(self,k1,k2,stat):
        data = None
        if self.ppRE.has_key(k1) and self.ppRE[k1].has_key(k2):
            data = numpy.array(self.ppRE[k1][k2], dtype='float64')
        elif self.ppRE.has_key(k2) and self.ppRE[k2].has_key(k1):
            data = numpy.array(self.ppRE[k2][k1], dtype='float64')
        else:
            return 0
        
        if stat == 'upper':
            return numpy.max(data)
        elif stat == 'lower':
            return numpy.min(data)
        elif stat == 'mean':
            return numpy.mean(data)  
        
    def get_ppDist(self,k):
        d = str(int(math.floor(k)))
        return self.ppDist[d]
    
#    def get_ppAccumPair(self,k1,k2,distance):
#        d = int(math.floor(distance))
#        if self.ppAccumPair.has_key(k1) and self.ppAccumPair[k1].has_key(k2):
#            return self.ppAccumPair[k1][k2][d]
#        else:
#            return self.ppAccumPair[k2][k1][d]
        
    """
        Create a random array substituting a previous one, it creates a random number to substitute each other number
    """
    def randomise_interaface(self,list,range):
        rand = []
        done = {}
        for pos in list:
            if done.has_key(pos):
                rand.append(done[pos])
            else:
                r = random.randint(0,(range-1))
                rand.append(r)
                done[pos] = r
        return rand
        
    """
        IO FUNCTIONS
        ------------
    """        
    def process_file_line(self, line, tag):
        
        if tag == 0:    return
        if tag == 1 and line.startswith(" PAIR"): self.assign_multiple_distances(dic=self.ppResidues, keys= line.split()[1],
                                                                                 values = line.split()[2:])
        if tag == 2 and line.startswith(" PAIR"): self.assign_multiple_distances(dic=self.ppEnvironment, keys= line.split()[1],
                                                                                 values = line.split()[2:])
        if tag == 3 and line.startswith(" AA-ENV"): self.assign_single_distance(dic=self.localppRE, key= line.split()[1],
                                                                                value = line.split()[2])
        if tag == 4 and line.startswith(" PAIR"): self.assign_multiple_distances(dic=self.ppRE, keys= line.split()[1],
                                                                                 values = line.split()[2:])
        if tag == 5 and line.startswith(" Distance"): self.assign_single_distance(dic=self.ppDist, key= line.split()[1],
                                                                                  value = line.split()[2])
#       if tag == 7 and line.startswith(" PAIR"): self.assign_multiple_distances(dic=self.ppAccumPair, keys= line.split()[1],
#                                                                                values = line.split()[2:])
        
    def assign_multiple_distances(self, dic, keys, values):
        
        dic.setdefault(keys.split(':')[0],{}).setdefault(keys.split(':')[1],[]).extend(values)
        
    def assign_single_distance(self, dic, key, value):
        
        dic[key] = value

    def two_keys_dic_multivalue_2string_array(self, dic, prev):
        
        lines = []
        for k1 in dic:
            for k2 in dic[k1]:
                line = prev + "%s:%s\t" %(k1, k2)
                line = line + "\t".join(dic[k1][k2])
                lines.append(line)
        return lines
                
    def one_key_dic_value_2string_array(self, dic, prev):
        
        lines = []
        for k in dic:
            line = prev + "\t".join((k,dic[k]))
            lines.append(line)
        return lines
    
    def ppResidues_2string_array(self):
        
        lines = []
        
        lines.append("PAIR-POTENTIALS OF RESIDUES")
        lines.extend(self.two_keys_dic_multivalue_2string_array(dic=self.ppResidues, prev = " PAIR "))
        lines.append("END")
        
        return lines
    
    def string_ppResidues(self): return "\n".join(self.ppResidues_2string_array())
    
    def ppEnvironment_2string_array(self):
        
        lines = []
        
        lines.append("PAIR-POTENTIALS OF ENVIRONMENTS")
        lines.extend(self.two_keys_dic_multivalue_2string_array(dic=self.ppEnvironment, prev = " PAIR "))
        lines.append("END")
        
        return lines
    
    def string_ppEnvironment(self): return "\n".join(self.ppEnvironment_2string_array())
    
    def localppRE_2string_array(self):
        
        lines = []
        
        lines.append("LOCAL PAIR-POTENTIALS OF RESIDUES AND ENVIRONMENTS")
        lines.extend(self.one_key_dic_value_2string_array(dic=self.localppRE, prev = " AA-ENV "))
        lines.append("END")
        
        return lines
    
    def string_localppRE(self): return "\n".join(self.localppRE_2string_array())
        
    def ppRE_2string_array(self):
        
        lines = []
        
        lines.append("PAIR-POTENTIALS OF RESIDUES AND ENVIRONMENTS")
        lines.extend(self.two_keys_dic_multivalue_2string_array(dic=self.ppRE, prev = " PAIR "))
        lines.append("END")
        
        return lines
    
    def string_ppRE(self): return "\n".join(self.ppRE_2string_array())
    
    def ppDist_2string_array(self):
        
        lines = []
        
        lines.append("STANDARD PAIR-POTENTIALS OF DISTANCES")
        lines.extend(self.one_key_dic_value_2string_array(dic=self.ppDist, prev = " Distance "))
        lines.append("END")
        
        return lines
    
    def string_ppDist(self): return "\n".join(self.ppDist_2string_array())
    
#    def ppAccumPair_2string_arra(self):
#        
#        lines = []
#        
#        lines.append("PAIR-POTENTIALS OF ACCUMULATED PAIRS")
#        lines.extend(self.two_keys_dic_multivalue_2string_array(dic=self.ppAccumPair, prev = " PAIR "))
#        lines.append("END")
#        
#        return lines        
#    
#    def string_ppAccumPair(self): return "\n".join(self.ppAccumPair_2string_arra())
              
    def __str__(self):
        
        lines = []
        
        lines.extend(self.ppResidues_2string_array())
        lines.extend(self.ppEnvironment_2string_array())
        lines.extend(self.localppRE_2string_array())
        lines.extend(self.ppRE_2string_array())
        lines.extend(self.ppDist_2string_array())
#       lines.extend(self.ppAccumPair_2string_arra())
        
        return "\n".join(lines)
        
        



import math
import string
import sys
import os

#ALPHABETS


protein_letters = 'ACDEFGHIKLMNPQRSTVWYXUBZ'
rna_letters = 'GAUC'
dna_letters = 'GATC'


#WEIGHTS

protein_weights = {'A': 89.09, 'C': 121.16, 'E': 147.13, 'D': 133.1, 'G': 75.07, 'F': 165.19, 'I': 131.18,
                   'H': 155.16,'K': 146.19, 'M': 149.21, 'L': 131.18, 'N': 132.12, 'Q': 146.15, 'P': 115.13,
                   'S': 105.09,'R': 174.2, 'T': 119.12, 'W': 204.23, 'V': 117.15, 'Y': 181.19, 'X': 0.0 ,
                   'U':0.0, 'B':0.0,'Z':0.0}

rna_weights = {'A': 363.0, 'C': 339.0, 'U': 340.0, 'G': 379.0}

dna_weights = {'A': 347.0, 'C': 323.0, 'T': 322.0, 'G': 363.0}

#COMPLEMENT

dna_complement = {'A': 'T', 'C': 'G', 'G': 'C', 'T': 'A'}
rna_complement = {'A': 'U', 'C': 'G', 'G': 'C', 'U': 'A'}
dna_transcribe = {'A': 'U', 'C': 'G', 'G': 'C', 'T': 'A'}
rna_transcribe = {'A': 'T', 'C': 'G', 'G': 'C', 'U': 'A'}


#CODON TABLES

rna_table = {'GUC': 'V', 'ACC': 'T', 'ACA': 'T', 'ACG': 'T', 'GUU': 'V', 'AAC': 'N', 'AGG': 'R', 'UGG': 'W', 'AGC': 'S', 'AUC': 'I', 'AGA': 'R', 'AAU': 'N', 'ACU': 'T', 'CAC': 'H', 'GUG': 'V', 'CCG': 'P', 'CCA': 'P', 'AGU': 'S', 'CCC': 'P', 'GGU': 'G', 'UCU': 'S', 'GCG': 'A', 'CGA': 'R', 'CAG': 'Q', 'CGC': 'R', 'UAU': 'Y', 'CGG': 'R', 'UCG': 'S', 'CCU': 'P', 'GGG': 'G', 'GGA': 'G', 'GGC': 'G', 'GAG': 'E', 'UCC': 'S', 'UAC': 'Y', 'CGU': 'R', 'GAA': 'E', 'AUA': 'I', 'GCA': 'A', 'CUU': 'L', 'UCA': 'S', 'AUG': 'M', 'CUG': 'L', 'AUU': 'I', 'CAU': 'H', 'CUA': 'L', 'GCC': 'A', 'AAA': 'K', 'AAG': 'K', 'CAA': 'Q', 'UUU': 'F', 'GAC': 'D', 'GUA': 'V', 'UGC': 'C', 'GCU': 'A', 'UGU': 'C', 'CUC': 'L', 'UUG': 'L', 'UUA': 'L', 'GAU': 'D', 'UUC': 'F'}

rna_table_back = {'A': 'GCU', 'C': 'UGU', None: 'UAA', 'E': 'GAG', 'D': 'GAU', 'G': 'GGU', 'F': 'UUU', 'I': 'AUU', 'H': 'CAU', 'K': 'AAG', 'M': 'AUG', 'L': 'UUG', 'N': 'AAU', 'Q': 'CAG', 'P': 'CCU', 'S': 'UCU', 'R': 'CGU', 'T': 'ACU', 'W': 'UGG', 'V': 'GUU', 'Y': 'UAU'}

rna_stop_codons = ['UAA', 'UAG', 'UGA']
rna_start_codons = ['UUG', 'CUG', 'AUG']



dna_table = {'CTT': 'L', 'ATG': 'M', 'ACA': 'T', 'ACG': 'T', 'ATC': 'I', 'ATA': 'I', 'AGG': 'R', 'CCT': 'P', 'AGC': 'S', 'AGA': 'R', 'ATT': 'I', 'CTG': 'L', 'CTA': 'L', 'ACT': 'T', 'CCG': 'P', 'AGT': 'S', 'CCA': 'P', 'CCC': 'P', 'TAT': 'Y', 'GGT': 'G', 'CGA': 'R', 'CGC': 'R', 'CGG': 'R', 'GGG': 'G', 'GGA': 'G', 'GGC': 'G', 'TAC': 'Y', 'CGT': 'R', 'GTA': 'V', 'GTC': 'V', 'GTG': 'V', 'GAG': 'E', 'GTT': 'V', 'GAC': 'D', 'GAA': 'E', 'AAG': 'K', 'AAA': 'K', 'AAC': 'N', 'CTC': 'L', 'CAT': 'H', 'AAT': 'N', 'CAC': 'H', 'CAA': 'Q', 'CAG': 'Q', 'TGT': 'C', 'TCT': 'S', 'GAT': 'D', 'TTT': 'F', 'TGC': 'C', 'TGG': 'W', 'TTC': 'F', 'TCG': 'S', 'TTA': 'L', 'TTG': 'L', 'TCC': 'S', 'ACC': 'T', 'TCA': 'S', 'GCA': 'A', 'GCC': 'A', 'GCG': 'A', 'GCT': 'A'}

dna_table_back = {'A': 'GCT', 'C': 'TGT', None: 'TAA', 'E': 'GAG', 'D': 'GAT', 'G': 'GGT', 'F': 'TTT', 'I': 'ATT', 'H': 'CAT', 'K': 'AAG', 'M': 'ATG', 'L': 'TTG', 'N': 'AAT', 'Q': 'CAG', 'P': 'CCT', 'S': 'TCT', 'R': 'CGT', 'T': 'ACT', 'W': 'TGG', 'V': 'GTT', 'Y': 'TAT'}

dna_stop_codons = ['TAA', 'TAG', 'TGA']
dna_start_codons = ['TTG', 'CTG', 'ATG']


# FUNCTIONS

def readframe(init,_sequence,start,stop,table):
  begin=False
  end=False
  proteins=[]
  seqprot=""
  for i in range(init,len(_sequence),3):
    codon=_sequence[i:i+3]
    if len(codon)<3:
      continue
    #print "ReadFrame %d Codon %s Seqprot %s \n"%(init,codon,seqprot)
    if codon in start and not begin:
      begin=True
      seqprot=""
    if begin and codon in stop:
      end=True
      begin=False
      if len(seqprot)>0:
        proteins.append(seqprot)
      continue
    if begin and not end:
      if codon in table:
        seqprot=seqprot+str(table[codon])
      else:
        raise ValueError("Codon %s not found in tables"%(codon))
  if len(seqprot)>0:
    proteins.append(seqprot)
  return proteins


def FASTA_iterator(fasta_filename):
 fd=open(fasta_filename,"r")
 seq=""
 name=""
 for line in fd:
   if line.startswith(">"):
    if len(name)>0 and len(seq)>0 :
        try:
          yield ProteinSequence(name,seq)
        except IncorrectSequenceLetter as e: #for python2.7 and up
          sys.stderr.write("%s"%(e))
          sys.stderr.write("Skip protein: %s\n"%(name))
    name=line.lstrip(">").strip()
    seq=""
   else:
    seq+=line.upper().strip()
 if len(name)>0 and len(seq)>0 :
  try:
    yield ProteinSequence(name,seq)
  except IncorrectSequenceLetter as e: #for python2.7 and up
    sys.stderr.write("%s\n"%e)
    sys.stderr.write("Skip protein: %s\n"%(name))
    
 fd.close()


# CLASSES

class IncorrectSequenceLetter(Exception):
  def __init__(self,letter,class_name):
    super(IncorrectSequenceLetter,self).__init__(
        "The _sequence items: '%s' are not found in the alphabet of class %s\n"%(letter,class_name))

class Sequence(object):
 alphabet=[]
 residue_mw={}
 def __init__(self,name=None,sequence=None):
  self._identifier=str(name)
  self._sequence=str(sequence)
  unknown=[w for w in self._sequence if w not in self.alphabet]
  if len(unknown)>0:
    word=",".join(set(unknown))
    raise IncorrectSequenceLetter(word,self.__class__.__name__)
 def __len__(self):
  return len(self._sequence)
 def __eq__(self,other):
  return self._sequence == other._sequence #This implies that we neglect different identifiers
  #return self._sequence == other._sequence and self.identifier == other.identifier
 def __ne__(self,other):
  return self._sequence != other._sequence
 def __le__(self,other):
  return self._sequence <= other._sequence
 def __ge__(self,other):
  return self._sequence >= other._sequence
 def __lt__(self,other):
  return self._sequence < other._sequence
 def __gt__(self,other):
  return self._sequence > other._sequence
 def __getitem__(self,item):
  return self._sequence[item]
 def __cmp__(self,other):
  if len(self._sequence) == len(other._sequence): return 0
  elif len(self._sequence) < len(other._sequence): return -1
  else: return 1
 def __hash__(self):
  #required to transform lists into set or dict (i.e. hashable elements)
  #this gives a unique id for a _sequence
  return self._sequence.__hash__()
 def __str__(self):
  return "I am a member of the Class %s"%(self.__class__.__name__)
 def __add__(self,other):
  return self.__class__(self.identifier,self._sequence+other._sequence)
 def count_tokens(self):
  return len(self._sequence)
 def get_identifier(self):
  return self._identifier
 def get_sequence(self):
  return self._sequence
 def get_mw(self):
  return sum(self.residue_mw.setdefault(aa,0) for aa in self._sequence)
 @property
 def identifier(self):
  return self._identifier
 @property
 def sequence(self):
  return self._sequence
 @property
 def mw(self):
  return sum(self.residue_mw.setdefault(aa,0) for aa in self._sequence)
 def has_sub_sequence(self,sub_sequence=None):
  if sub_sequence is None:
    raise ValueError("No sub_sequence was given")
  else:
    return sub_sequence.get_sequence() in self._sequence

class NucleotideSequence(Sequence):
 complement={}
 table={}
 stop=[]
 start=[]
 def get_complement(self):
  na= "".join([self.complement[u] for u in self._sequence])
  id=self.identifier
  return self.__class__(id,na)
 def translate(self):
  id=self.identifier
  proteins={}
  #proteins=set()
  for init in range(3):
    proteins.setdefault(init,readframe(init,self._sequence,self.start,self.stop,self.table))
    #proteins.update(readframe(init,self._sequence,self.start,self.stop,self.table))
  return proteins

class DNASequence(NucleotideSequence):
 alphabet=dna_letters
 residue_mw=dna_weights
 complement=dna_complement
 table=dna_table
 start=dna_start_codons
 stop=dna_stop_codons
 transcript=dna_transcribe
 def transcribe(self):
  trans="".join([self.transcript[u] for u in self.get_sequence()])
  id=self.get_identifier()
  return RNASequence(id,trans)

class RNASequence(NucleotideSequence):
 alphabet=rna_letters
 residue_mw=rna_weights
 complement=rna_complement
 table=rna_table
 start=rna_start_codons
 stop=rna_stop_codons
 transcript=rna_transcribe
 def reverse_transcribe(self):
  trans="".join([self.transcript[u] for u in self.get_sequence()])
  id=self.get_identifier()
  return DNASequence(id,trans)

class ProteinSequence(Sequence):
 alphabet=protein_letters
 residue_mw=protein_weights
 def reverse_translate_to_RNA(self):
  rnaseq= "".join([rna_table_back[u] for u in self.get_sequence()])
  id=self.get_identifier()
  return RNASequence(id,rnaseq)
 def reverse_translate_to_DNA(self):
  dnaseq= "".join([dna_table_back[u] for u in self.get_sequence()])
  id=self.get_identifier()
  return DNASequence(id,dnaseq)




"""
PDB:
"""
PDBftp   = {'address'    : 'ftp.wwpdb.org', 
            'structures' : '/pub/pdb/data/structures/divided/pdb/',
            'derived'    : '/pub/pdb/derived_data/index/',
            'resolution' : 'resolu.idx',
            'show'       : 'http://www.pdb.org/pdb/home/home.do'}
PDBrsync = {'port':   '33444', 
            'address':'rsync.wwpdb.org::ftp/data/structures/divided/pdb/'}

"""
PDBeChem:
"""
PDBeChemftp = {'global':'ftp://ftp.ebi.ac.uk/pub/databases/msd/pdbechem/files/mmcif.tar.gz',
			   'single':'ftp://ftp.ebi.ac.uk/pub/databases/msd/pdbechem/files/mmcif/',
               'show'  :'ftp://ftp.ebi.ac.uk/pub/databases/msd/pdbechem/'}

"""
GO:
"""
GOftp       = {'source':'ftp://ftp.geneontology.org/pub/go/ontology/go-simple.obo',
               'show'  : 'http://www.geneontology.org/'}
GOnamespace = {'biological_process':'B','cellular_component':'C','molecular_function':'M',
               'B':'biological_process','C':'cellular_component','M':'molecular_function'}

"""
TAXID:
"""
taxIDftp = {'global':'ftp://ftp.ncbi.nih.gov/pub/taxonomy/taxdmp.zip',
            'show'  :'http://www.ncbi.nlm.nih.gov/taxonomy'}

"""
UNIPROT:
"""
Uniprotftp = {'swissprot':'ftp://ftp.uniprot.org/pub/databases/uniprot/current_release/knowledgebase/complete/uniprot_sprot.dat.gz', 
              'trembl'   :'ftp://ftp.uniprot.org/pub/databases/uniprot/current_release/knowledgebase/complete/uniprot_trembl.dat.gz',
              'show'     :'http://www.uniprot.org/'}

"""
ENZYME:
"""
Enzymeftp = {'dat'  : 'ftp://ftp.expasy.org/databases/enzyme/enzyme.dat',
             'cls'  : 'ftp://ftp.expasy.org/databases/enzyme/enzclass.txt',
             'show' : 'http://enzyme.expasy.org/'}

"""
DRUGBANK:
"""
drugBankftp = {'show'    : 'http://www.drugbank.ca/',
               'main'    : 'http://www.drugbank.ca/system/downloads/current/drugbank.xml.zip',
               'targets' : 'http://www.drugbank.ca/system/downloads/current/all_target_ids_all.csv.zip'}

"""
SCOP:
"""
SCOPftp = {'show' : 'http://scop.mrc-lmb.cam.ac.uk/scop',
           'desc' : 'http://www.mrc-lmb.cam.ac.uk/agm/pre-scop/parseable/dir.des.scop.txt_MASTER.out',
           'rel'  : 'http://www.mrc-lmb.cam.ac.uk/agm/pre-scop/parseable/dir.cla.scop.txt_MASTER.out'}

"""
PDBTM:
"""
PDBTMftp = {'svn' : 'http://w2.enzim.hu/pdbtm/pdbtm',
            'show': 'http://pdbtm.enzim.hu/'}

"""
INCLUDES
"""
from PDBlink      import PDBlink
from PDBeChemlink import PDBeChemlink, PDBeChem
from GOlink       import GOlink, GOterm
from TaxIDlink    import TaxIDlink, TaxID
from Uniprotlink  import Uniprotlink, Uniprot
from Enzymelink   import Enzymelink, Enzyme
from DrugBanklink import DrugBanklink, Drug
from SCOPlink     import SCOPlink
from PDBTMlink    import PDBTMlink, TM





import sys
import os
import argparse
import gzip
import re

from SBI.beans           import Path

from archdb.src import Subclass, Cclass, Loop
from archdb.src import start_transaction, end_transaction


# User Options
def options(*args, **kwds):
    parser = argparse.ArgumentParser(prog = 'MCS2SQL',  formatter_class = argparse.ArgumentDefaultsHelpFormatter)
    parser.add_argument('-d', '--database',     dest     = "database",
                        action = "store",       required = True,  help = "MCL output dir.")
    parser.add_argument('-q', '--looplist',     dest     = "looplist",
                        action = "store",       required = True,  help = "List of loops by order in PDB and length of first ss.")
    parser.add_argument('-s', '--sql',          dest     = "sqlfile",
                        action = "store",       required = True,  help = "Output SQL dir")
    parser.add_argument('-v', '--verbose',      dest     = "verbose",
                        action = "store_true",  default  = False, help = "Verbose Mode.")
    options = parser.parse_args()
    return options


def MCL2SQL(database, looplist, sqlfile, verbose):

    Path.mkdir(sqlfile)
    grupfiles = {}
    for mclfile in Path.list_files(database):
        subclasstype = os.path.split(mclfile)[-1].split('.')[1]
        grupfiles.setdefault(subclasstype, []).append(mclfile)

    for subclasstype in grupfiles:
        classification = Cclass(subclasstype)
        loops = readlist(looplist, subclasstype)
        sql_fd = gzip.open(os.path.join(sqlfile, subclasstype + '.sql.gz'), 'wb')
        sql_fd.write(start_transaction())
        for mclfile in sorted(grupfiles[subclasstype]):
            subclassrange = os.path.split(mclfile)[-1].split('.')[2]
            if verbose:
                sys.stderr.write("Retrieving data for subclass {0} {1}...\n".format(subclasstype, subclassrange))
            sql_in = open(mclfile)
            read   = False
            for line in sql_in:
                dataline = line.rstrip('\n')
                #SKIP LINES
                if line.startswith('==') or line.startswith('***') or len(line.strip()) == 0 or line.startswith('---- P R O T E I N    C O D E  ----'):
                    continue
                if line.startswith('CONSENSUS & MULTIPLE ALIGNEMENT IN THE'):
                    data   = line.split(':')[-1].strip().split()
                    classification.subclasses = Subclass(tuple([data[0].strip(), data[3].strip()]), data[4], subclassrange)
                    workscls = classification.lastsubclass
                    read = True
                    continue
                if line.startswith('GLOBAL STATISTICS'):
                    read = False
                    continue
                if read:
                    if line.startswith('        SEQUENCE   ALIGNEMENT                           :'):   parse_mode, counter = 'P', 0
                    elif line.startswith('       ACCESSIBLE SURFACE ALIGNEMENT                    :'): parse_mode, counter = 'E', 0
                    elif line.startswith('           RAMACHANDRAN                                 :'): parse_mode, counter = 'R', 0
                    elif line.startswith('        SECONDARY STRUCTURE                             :'): parse_mode, counter = 'S', 0
                    elif line.startswith('--------- CONSENSUS THORNTON       :'):   workscls.add_consensus(dataline, 'MCL', loops)
                    elif line.startswith('--------- CONSENSUS TOPOLOGY'):           workscls.add_topology(dataline, 'MCL')
                    elif line.startswith('CENTROIDE POLAR COORD.   :'):             workscls.add_coordinates(dataline)
                    elif line.startswith('--------- RAMACHANDRAN PATTERN     :'):   workscls.ram_pat = re.sub('\(X\)','',dataline.split(':')[1].strip().strip('.'))
                    elif line.startswith('--------- SEQUENCE  PATTERN        :'):   workscls.seq_pat = re.sub('\(X\)','',dataline.split(':')[1].strip().strip('.'))
                    elif line.startswith('--------- BURIAL    PATTERN        :'):   workscls.exp_pat = re.sub('\(X\)','',dataline.split(':')[1].strip().strip('.'))

                    elif line.startswith('                             ') and len(dataline) < 400:
                        if parse_mode == 'P':
                            workscls.loops = Loop(info = dataline)
                        if parse_mode == 'E':
                            workscls.loops[counter].add_surface(info = dataline)
                            counter += 1
                        if parse_mode == 'R':
                            workscls.loops[counter].add_ramachandran(info = dataline)
                            counter += 1
                        if parse_mode == 'S':
                            workscls.loops[counter].add_secondary_str(info = dataline)
                            counter += 1

        sql_fd.write(classification.toSQL('MCL'))

        sql_in.close()
        sql_fd.write(end_transaction())
        sql_fd.close()

    if verbose:
        sys.stderr.write("End execution.\n")


def readlist(looplist, cstype):
    loops = {}
    fd = open(looplist)
    for l in fd:
        d = l.strip().split()

        if d[1] == cstype:
            n = d[0].split('_')
            key = tuple(['_'.join(n[0:2]), n[2]])
            loops[key] = int(d[2])
    fd.close()
    return loops

if __name__ == "__main__":
    options = options()
    MCL2SQL(options.database, options.looplist, options.sqlfile, options.verbose)



import sys
import os
import re


if len(sys.argv)>2:
  fd=open(sys.argv[1],"r")
  outdir=sys.argv[2]
elif len(sys.argv)==2:
  fd=open(sys.argv[1],"r")
  outdir="./"
else:
  print "Run as parse_list.py 'input_list' [FOLDER NAME]"
  print "Missing input list"
  exit(0)

pair={}
for line in fd:
   a,b=line.strip().split()
   x=a.split("_")[0]
   y=b.split("_")[0]
   if pair.has_key((x,y)) or pair.has_key((y,x)):
    if pair.has_key((x,y)):
     pair.setdefault((x,y),set()).add((a,b))
    else:
     pair.setdefault((y,x),set()).add((b,a))
   else:
     pair.setdefault((x,y),set()).add((a,b))
fd.close()

if not os.path.exists(outdir): os.makedirs(outdir)
  
for wt,forms in pair.iteritems():
   output=open(os.path.join(outdir,wt[0]+"_"+wt[1]+".ppi"),"w")
   for p,q in forms:
    output.write("%s\t%s\n"%(p,q))
   
  




"""
Complex

author: jbonet
date:   02/2014

@oliva's lab
"""
from .   import PPInterface, PNInterface, PHInterface
from SBI import SBIglobals

class Complex(object):
    def __init__(self, pdb, biomolecule = False, PPI = True, PPI_type = "cb",  PPI_distance = 12,
                                                 PNI = True, PNI_type = "min", PNI_distance = 8,
                                                 PHI = True, PHI_type = "min", PHI_distance = 6):
        self._pdb          = pdb
        self._biomolecule  = biomolecule

        self._PPIdo        = PPI
        self._PPInterface  = {}
        self._PPIblocked   = {}
        self._PPI_type     = PPI_type
        self._PPI_distance = PPI_distance

        self._PNIdo        = PNI
        self._PNInterface  = {}
        self._PNIblocked   = {}
        self._PNI_type     = PNI_type
        self._PNI_distance = PNI_distance

        self._PHIdo        = PHI
        self._PHInterface  = {}
        self._PHIblocked   = {}
        self._PHI_type     = PHI_type
        self._PHI_distance = PHI_distance

        self._build()

    #
    # ATTRIBUTES
    #
    @property 
    def pdb(self):          return self._pdb

    @property 
    def biomolecule(self):  return self._biomolecule

    @property 
    def PPInterfaces(self): return self._PPInterface.values()

    @property 
    def PNInterfaces(self): return self._PNInterface.values()

    @property 
    def PHInterfaces(self): return self._PHInterface.values()

    #
    # PRIVATE FUNCTIONS
    #
    def _build(self):
        structures = []
        count      = 1

        if not self._biomolecule: 
            structures.append(self._pdb)
        else:
            SBIglobals.alert('debug', self, 'Building biomolecules')   
            structures.extend(self.pdb.apply_biomolecule_matrices(keepchains = True, water = False))

        SBIglobals.alert('debug', self, 'Analyzing Interfaces of {0:03} biomolecules'.format(len(structures)))

        for biom in structures:
            SBIglobals.alert('debug', self, 'Analyzing Biomolecule {0:03} out of {1:03}'.format(count, len(structures)))

            protein_chains = []
            protein_pgeoms = []
            protein_hgeoms = [] 
            for p in biom.proteins:
                protein_chains.append(p)
                protein_pgeoms.append(p.geometric_center(structure = True,  hetero = False, water = False, by_residue = True))
                protein_hgeoms.append(p.geometric_center(structure = False, hetero = True,  water = False, by_residue = True))
            nucleotide_chains = []
            nucleotide_ngeoms = []
            nucleotide_hgeoms = []
            for n in biom.nucleotides:
                nucleotide_chains.append(n)
                nucleotide_ngeoms.append(n.geometric_center(structure = True,  hetero = False, water = False, by_residue = True))
                nucleotide_hgeoms.append(n.geometric_center(structure = False, hetero = True,  water = False, by_residue = True))

            total_chains = 0
            if self._PPIdo: total_chains += len(protein_chains)
            if self._PNIdo: total_chains += len(nucleotide_chains)

            SBIglobals.alert('debug', self, '\tBiomolecule has {0:03} chains -> {1:03} max. Interfaces'.format(total_chains,(total_chains*(total_chains-1))/2))

            for i in xrange(len(protein_chains)):
                for j in xrange(i+1, len(protein_chains)):
                    self._add_PPI(protein_chains[i], protein_chains[j], protein_pgeoms[i], protein_pgeoms[j])
                for j in xrange(len(protein_chains)):
                    if i != j:
                        self._add_PHI(protein_chains[i], protein_chains[j], protein_pgeoms[i], protein_hgeoms[j])
                for j in xrange(len(nucleotide_chains)):
                    self._add_PNI(protein_chains[i], nucleotide_chains[j], protein_pgeoms[i], nucleotide_ngeoms[j])
                    self._add_PHI(protein_chains[i], nucleotide_chains[j], protein_pgeoms[i], nucleotide_hgeoms[j])

    def _add_PPI(self, chain1, chain2, geom1, geom2):
        if self._PPIdo:
            ppi_id = PPInterface.test_identifier(chain1, chain2)
            SBIglobals.alert('debug', self, 'Analyzing Protein-Protein Interface {0} for {1.chain} - {2.chain}'.format(ppi_id, chain1, chain2))
            if ppi_id not in self._PPIblocked or self._PPIblocked[ppi_id] < 2:
                SBIglobals.alert('debug', self, '\tInterface is NOT blocked')
                self._PPIblocked.setdefault(ppi_id, 0)
                ppi = PPInterface(chain1, chain2, self._PPI_type, self._PPI_distance, geom1, geom2)
                if ppi_id not in self._PPInterface:
                    SBIglobals.alert('debug', self, '\tAdding new Interface')
                    self._PPInterface[ppi_id] = ppi
                else:
                    SBIglobals.alert('debug', self, '\tUpdating Interface')
                    l = len(self._PPInterface[ppi_id])
                    self._PPInterface[ppi_id] + ppi
                    L = len(self._PPInterface[ppi_id])
                    if l == L:
                        SBIglobals.alert('debug', self, '\t\tInterface does NOT give any new contact')
                        self._PPIblocked[ppi_id] += 1
                    else:
                        SBIglobals.alert('debug', self, '\t\tInterface GIVES new contacts')
            else:
                SBIglobals.alert('debug', self, '\tInterface IS blocked')

    def _add_PNI(self, chain1, chain2, geom1, geom2):
        if self._PNIdo:
            pni_id = PNInterface.test_identifier(chain1, chain2)
            SBIglobals.alert('debug', self, 'Analyzing Protein-Nucleotide Interface {0} for {1.chain} - {2.chain}'.format(pni_id, chain1, chain2))
            if pni_id not in self._PNIblocked or self._PNIblocked[pni_id] < 2:
                SBIglobals.alert('debug', self, '\tInterface is NOT blocked')
                self._PNIblocked.setdefault(pni_id, 0)
                pni = PNInterface(chain1, chain2, self._PNI_type, self._PNI_distance, geom1, geom2)
                if pni_id not in self._PNInterface:
                    SBIglobals.alert('debug', self, '\tAdding new Interface')
                    self._PNInterface[pni_id] = pni
                else:
                    SBIglobals.alert('debug', self, '\tUpdating Interface')
                    l = len(self._PNInterface[pni_id])
                    self._PNInterface[pni_id] + pni
                    L = len(self._PNInterface[pni_id])
                    if l == L:
                        SBIglobals.alert('debug', self, '\t\tInterface does NOT give any new contact')
                        self._PNIblocked[pni_id] += 1
                    else:
                        SBIglobals.alert('debug', self, '\t\tInterface GIVES new contacts')
            else:
                SBIglobals.alert('debug', self, '\tInterface IS blocked')

    def _add_PHI(self, chain1, chain2, geom1, geom2):
        if self._PHIdo:
            phi_id = PHInterface.test_identifier(chain1, chain2)
            SBIglobals.alert('debug', self, 'Analyzing Protein-Heteroatom Interface {0} for {1.chain} - {2.chain}'.format(phi_id, chain1, chain2))
            if phi_id not in self._PHIblocked or self._PHIblocked[phi_id] < 2:
                SBIglobals.alert('debug', self, '\tInterface is NOT blocked')
                self._PHIblocked.setdefault(phi_id, 0)
                phi = PHInterface(chain1, chain2, self._PHI_type, self._PHI_distance, geom1, geom2)
                if phi_id not in self._PHInterface:
                    SBIglobals.alert('debug', self, '\tAdding new Interface')
                    self._PHInterface[phi_id] = phi
                else:
                    SBIglobals.alert('debug', self, '\tUpdating Interface')
                    l = len(self._PHInterface[phi_id])
                    self._PHInterface[phi_id] + phi
                    L = len(self._PHInterface[phi_id])
                    if l == L:
                        SBIglobals.alert('debug', self, '\t\tInterface does NOT give any new contact')
                        self._PHIblocked[phi_id] += 1
                    else:
                        SBIglobals.alert('debug', self, '\t\tInterface GIVES new contacts')
            else:
                SBIglobals.alert('debug', self, '\tInterface IS blocked')

    #
    # OVERRIDE DEFAULT METHODS
    #
    def __str__(self):
        data = []
        data.append("Contacts for {0}".format(self.pdb.id))
        for ppi in self._PPInterface:
            if len(self._PPInterface[ppi]) > 0:
                data.append(self._PPInterface[ppi].toString(True))
        for pni in self._PNInterface:
            if len(self._PNInterface[pni]) > 0:
                data.append(self._PNInterface[pni].toString(True))
        for phi in self._PHInterface:
            if len(self._PHInterface[phi]) > 0:
                data.append(self._PHInterface[phi].toString(True))
        return "\n".join(data)








import sys
import argparse
import os
import ConfigParser
import gzip
import shutil
import subprocess

# Add "." to sys.path #
src_path =  os.path.abspath(os.path.dirname(__file__))+"/../../scripts/"
sys.path.append(src_path)

# Read configuration file
config = ConfigParser.ConfigParser()
config_file = os.path.join(src_path, 'config.ini')
config.read(config_file)

# Add SBI library path to sys.path
src_path = config.get('Paths', 'sbi_library_path')
sys.path.append(src_path)

# Add functions path to sys.path
src_path = config.get('Paths', 'functions_path')
sys.path.append(src_path)

# Import functions

from SBI.structure import PDB
import SBI.structure.contacts as cn
from functions import *



def main():

    options = parse_user_arguments()
    pdb2pin(options)

def parse_user_arguments(*args, **kwds):

    parser = argparse.ArgumentParser(
        description = "Create a network of PPI's extracted from PDB complexes",
        epilog      = "@oliva's lab 2015")
    parser.add_argument('-i','--list_pdb_files',dest='listfile',action = 'store',
                        help = 'List of PDB files with the address location')
    parser.add_argument('-o','--output_names',dest='out',action = 'store', default='output',
                        help = 'Output rootname for files with nodes and edges(default is output)')
    parser.add_argument('--PPI_distance',dest='PPI_distance',action = 'store', type=float, default=12.0,
                        help = 'Threshold distance to calculate contacts')
    parser.add_argument('--PPI_type',dest='PPI_type',action = 'store', default='cb',
                        help = 'Type of contact: ca, cb or min')
    parser.add_argument('-v','--verbose',dest='show',action = 'store_true',
                        help = 'Verbose execution ')
    options=parser.parse_args()

    return options

def pdb2pin(options):

 filelist=options.listfile
 rootname=options.out
 verbose=options.show
 PPI_distance = options.PPI_distance
 PPI_type = options.PPI_type
 
 edges=[]
 nodes=[]

 if fileExist(filelist):
  for line in parse_list_file(filelist):
   pdb=PDB(line)
   complexes=cn.Complex(pdb,PPI_distance = PPI_distance, PPI_type = PPI_type)
   for pair in complexes.PPInterfaces:
     #if not pair.is_empty:
     if len(pair.contacts)>0:
      edges.append((pair.interactor_id,pair.protein_id))
      if verbose: sys.stdout.write("Add {0:s}\t{1:s}\n".format(pair.interactor_id,pair.protein_id))
  nodes=get_nodes(edges)
 else:
  sys.stderr.write("Missing list of PDB files\n")
  exit(0)

 ppi=open(rootname+".ppi","w")
 nds=open(rootname+".dat","w")
 for (x,y) in edges:
  ppi.write("%s\t%s\n"%(x,y))
 for x in nodes:
  nds.write("%s\n"%(x))



if  __name__ == "__main__":
    main()






import sys, os
from collections import Counter

class Interaction(object):
    '''
    This class represent an interaction between two structures
    '''
 
    def __init__(self, structure1, structure2):
        '''
        Constructor
        '''
        self.structure1 = structure1
        self.structure2 = structure2
        self.positive_signatures = []
        self.negative_signatures = []

    # GETTERS #

    def get_structure1(self):
        return self.structure1

    def get_structure2(self):
        return self.structure2
   
    def get_structures(self):
        return self.structure1, self.structure2

    def get_positive_signatures(self):
        return self.positive_signatures

    def get_negative_signatures(self):
        return self.negative_signatures

    # SETTERS #

    def set_positive_signatures(self, positive_signatures):
        self.positive_signatures = positive_signatures

    def set_negative_signatures(self, negative_signatures):
        self.negative_signatures = negative_signatures

    # CLASS METHODS #

    def get_signatures_by_location(self, min_coverage = 0.8, c_type = 'CB', max_distance = 12, max_pval = 1):
        '''
        Get the signatures in the interface and the total combination of signatures
        NOTE: We assume the signatures are ordered from better to worst pValue
        NOTE: We assume structure 1 contains the loops of the first protein signature: (str1 loops, str2loops, pValue)
        NOTE: I: interface
              E: external
              P: partial
        '''
        # Get interacting loops of each strcuture #
        int_loops_structure1 = [i.get_id() for i in self.get_structure1().get_interacting_loops(self.get_structure2(), min_coverage, c_type, max_distance)]
        int_loops_structure2 = [i.get_id() for i in self.get_structure2().get_interacting_loops(self.get_structure1(), min_coverage, c_type, max_distance)]
        nonint_loops_structure1 = [i.get_id() for i in self.get_structure1().get_noninteracting_loops(self.get_structure2(), min_coverage, c_type, max_distance)]
        nonint_loops_structure2 = [i.get_id() for i in self.get_structure2().get_noninteracting_loops(self.get_structure1(), min_coverage, c_type, max_distance)]
        # Init signatures datatype #
        signatures = {'I': ([], [], len(int_loops_structure1)*len(int_loops_structure2)), 
                      'E' : ([], [], len(nonint_loops_structure1)*len(nonint_loops_structure1)),
                      'P'  : ([], [], len(int_loops_structure1)*len(nonint_loops_structure2)+len(int_loops_structure2)*len(nonint_loops_structure1))}
        # Iterate over signatures # 
        for positive_signature in self.get_positive_signatures():
            in_int_structure1 = not Counter(positive_signature[0])-Counter(int_loops_structure1)
            in_int_structure2 = not Counter(positive_signature[1])-Counter(int_loops_structure2)
            in_noint_structure1 = not Counter(positive_signature[0])-Counter(nonint_loops_structure1)
            in_noint_structure2 = not Counter(positive_signature[1])-Counter(nonint_loops_structure2)
            if in_int_structure1 and in_int_structure2 and positive_signature[2] <= max_pval:
                signatures['I'][0].append(positive_signature)
            if in_noint_structure1 and in_noint_structure2 and positive_signature[2] <= max_pval:
                signatures['E'][0].append(positive_signature)
            if in_noint_structure1 and in_int_structure2 and positive_signature[2] <= max_pval:
                signatures['P'][0].append(positive_signature)
            elif in_int_structure1 and in_noint_structure2 and positive_signature[2] <= max_pval:
                signatures['P'][0].append(positive_signature)
        for negative_signature in self.get_negative_signatures():
            in_int_structure1 = not Counter(negative_signature[0])-Counter(int_loops_structure1)
            in_int_structure2 = not Counter(negative_signature[1])-Counter(int_loops_structure2)
            in_noint_structure1 = not Counter(negative_signature[0])-Counter(nonint_loops_structure1)
            in_noint_structure2 = not Counter(negative_signature[1])-Counter(nonint_loops_structure2)
            if in_int_structure1 and in_int_structure2 and negative_signature[2] <= max_pval:
                signatures['I'][1].append(negative_signature)
            if in_noint_structure1 and in_noint_structure2 and negative_signature[2] <= max_pval:
                signatures['E'][1].append(negative_signature)
            if in_noint_structure1 and in_int_structure2 and negative_signature[2] <= max_pval:
                signatures['P'][1].append(negative_signature)
            elif in_int_structure1 and in_noint_structure2 and negative_signature[2] <= max_pval:
                signatures['P'][1].append(negative_signature)
        # Return signatures and total combination
        return signatures
        
    def get_interacting_residues(self, c_type = 'CB', max_distance = 12, uniq=True, warnings=False):
        '''
        Get a list of the interacting residues of both structures and distances
        '''
        return self.get_structure1().get_interacting_residues(self.get_structure2(), c_type, max_distance, uniq, warnings)

    def get_noninteracting_residues(self, c_type = 'CB', max_distance = 12, warnings=False):
        '''
        Get a list of the noninteracting residues of both structures
        '''
        interacting_residues = self.get_structure1().get_interacting_residues(self.get_structure2(), c_type, max_distance, True, warnings)
        return list(set(self.get_structure1().get_residues()).difference(set(interacting_residues[0]))), list(set(self.get_structure2().get_residues()).difference(set(interacting_residues[1])))

    def get_opposite_interacting_residues(self, size_str1 = 1, size_str2 = 1, c_type = 'CB', max_distance = 12, warnings=False):
        '''
        Get a list of residues in the opposite face of the interaction of both structures
        '''
        return self.get_structure1().get_opposite_interacting_residues(self.get_structure2(), size_str1, size_str2, c_type, max_distance, warnings)

    def __str__(self):
        '''
        Return the interacting residues
        '''
        interacting_residues = self.get_structure1().get_interacting_residues(self.get_structure2(), uniq=True)
        interactionPDB = '%s\n' % self.get_structure1().get_id()
        for residue in interacting_residues[0]:
            interactionPDB += residue.__str__()
        interactionPDB = '%s\n' % self.get_structure2().get_id()
        for residue in interacting_residues[1]:
            interactionPDB += residue.__str__()
        return interactionPDB



from SBI.databases import Drug

tables = {'drg'      : 'drugBank',
          'drg_cat'  : 'drugBank_category',
          'drg_ext'  : 'drugBank_external_identifiers',
          'drg_grp'  : 'drugBank_group',
          'drg_san'  : 'drugBank_secondary_accession_number',
          'drg_syn'  : 'drugBank_synonym',
          'drg_tar'  : 'drugBank_target', 
          'drg_act'  : 'drugBank_target_action',
          'uni'      : 'uniprot_entry2accession',
          'uniprot'  : 'uniprot'}

class Drug(Drug):
    def __init__(self, inline):
        super(Drug, self).__init__(inline = inline)

    @staticmethod
    def preuniprotdeleted():
        return "INSERT INTO {0} (entry, source) VALUES ('FAKE_PROTEIN', 'swissprot');\n".format(tables['uniprot'])
    @staticmethod
    def afteruniprotdeleted():
        command  = "DELETE FROM {0} WHERE uniprot = 'FAKE_PROTEIN';".format(tables['drg_act'])
        command += "DELETE FROM {0} WHERE uniprot = 'FAKE_PROTEIN';".format(tables['drg_tar'])
        command += "DELETE FROM {0} WHERE entry = 'FAKE_PROTEIN';".format(tables['uniprot'])
        return command

    def toSQL(self):
        h  =     self.dproperty["Hydrophobicity"]        if "Hydrophobicity"    in self.dproperty else 'NULL'
        i  =     self.dproperty["Isoelectric Point"]     if "Isoelectric Point" in self.dproperty else 'NULL'
        mw =     self.dproperty["Molecular Weight"]      if "Molecular Weight"  in self.dproperty else 'NULL'
        mf = "'"+self.dproperty["Molecular Formula"]+"'" if "Molecular Formula" in self.dproperty else 'NULL'
        cas_number          = "'"+self.cas+"'"                     if self.cas  != '' else 'NULL'
        description         = "'"+self.desc.replace("'",r'\'')+"'" if self.desc != '' else 'NULL'
        name                = self.name.replace("'",r'\'')

        command = ""
        command += "INSERT INTO {0}(drugbank_id,name,type,cas_number,description,hydrophobicity,isoelectric_point,molecular_weight,molecular_formula) ".format(tables['drg'])
        command += "VALUES ('{0.dbid}','{7}','{0.type}',{1},{2},{3},{4},{5},{6});\n".format(self, cas_number,description,h,i,mw,mf, name)
        for c in self.categories:
            command += "INSERT INTO {0} (drugbank_id, drugbank_category) VALUES ('{1.dbid}','{2}');\n".format(tables['drg_cat'], self, c)
        for e in self.external:
            command += "INSERT INTO {0} (drugbank_id, identifier, resource) VALUES ('{1.dbid}','{2[1]}','{2[0]}');\n".format(tables['drg_ext'],self,e)
        for g in self.groups:
            command += "INSERT INTO {0} (drugbank_id, drugbank_group) VALUES ('{1.dbid}','{2}');\n".format(tables['drg_grp'], self, g)
        for s in self.secondary_accession_numbers:
            command += "INSERT INTO {0} (drugbank_id, secondary_accession_number) VALUES ('{1.dbid}','{2}');\n".format(tables['drg_san'], self, s)
        for y in self.synonym:
            command += "INSERT INTO {0} (drugbank_id,drugbank_synonym) VALUES ('{1.dbid}','{2}');\n".format(tables['drg_syn'], self, y.replace("'",r'\''))
        for t in self.partners:
            if t[1] != ('',''):
                command += "SET @uni = (SELECT COALESCE((SELECT entry FROM uniprot WHERE entry = '{0[1]}'),(SELECT entry FROM uniprot_entry2accession WHERE accession='{0[0]}' LIMIT 1),'FAKE_PROTEIN'));\n".format(t[1])
                known = 'yes' if t[-1] == 'yes' else 'unknown' if t[-1] == 'unknown' else 'no'
                command += "INSERT INTO {0} (drugbank_id,uniprot,known_action,target_type) VALUES ('{1.dbid}',@uni,'{2}','{3}');\n".format(tables['drg_tar'], self, known, t[0])

                for i in range(2,len(t)):
                    if t[i] != 'yes' and t[i] != 'unknown':
                        command += "INSERT INTO {0} (drugbank_id,uniprot,action) VALUES ('{1.dbid}',@uni, '{2}');\n".format(tables['drg_act'], self, t[i])
        return command




'''
\usepackage{hyperref}

@file: butler.py

@author: Jaume Bonet
@mail:   jaume.bonet@gmail.com
@date:   2014

@ [oliva's lab](http://sbi.imim.es)

@class: Singleton
@class: Butler

\section{Description}
The \href{}{Singleton} object ensures that only one instance of a given object
is created. This is required as only one \href{}{Buttler} can go on around the
library consistently.

The \href{}{Buttler} class will create an instance that will move around the
library managing different aspects of it, such as overwriting permissions on
files or the logging.
'''
import sys
import os
import traceback
import datetime
import time
import logging
import inspect

from .singleton import Singleton


class Butler(object):
    '''
    Designed to work through all the \textbf{SBI library}.

    It contains several parameters that will control:
        \begin{itemize}
            \item the amount of data shown to the user during the execution of
                  \textbf{SBI} subroutines
            \item the file overwrite settings in a execution as long as it is
                  managed through a \href{}{File} object
        \end{itemize}

    Regarding the looging, it provides functions similar to those of a logger.
    '''
    __metaclass__   = Singleton

    _LOGNAME        = 'SBILOG'
    _GENERAL_FORMAT = '[{0}] ' + '%(asctime)s - %(levelname)-7.7s - %(message)s'
    _TIME_FORMAT    = '%Y-%m-%d %H:%M'

    def __init__(self):
        self._report = [
            False,  # Verbose
            False,  # Debug
            False,  # Deep Debug
            True,   # Warning
            True    # Error
        ]

        self._ios = [
            False,  # Overwrite
            False   # STDOUT
        ]

        self._fd = logging.getLogger(self._LOGNAME)
        self._fd.setLevel(logging.NOTSET)
        self._fd.addHandler(logging.NullHandler())

    ##############
    # ATTRIBUTES #
    ##############

    @property
    def verbose(self):
        '''
        Minimum level of progress info.
        Mainly, to inform of advance.

        @return:  {Boolean}
        @default: _False_
        '''
        return self._report[0]

    @verbose.setter
    def verbose(self, value):
        '''
        @param: {Boolean}
        '''
        self._report[0] = bool(value)

    @property
    def debug(self):
        '''
        Medium level of progress info.
        Use to better understand the process behind a particular function.

        @return:  {Boolean}
        @default: _False_
        '''
        return self._report[1]

    @debug.setter
    def debug(self, value):
        '''
        @param: {Boolean}
        '''
        value = bool(value)
        if value:
            self._report[0:2] = [value]*2
        else:
            self._debug       = value

    @property
    def deepdebug(self):
        '''
        Maximum level of progress info.
        To check on processes that are going to be highly repeated.

        @return:  {Boolean}
        @default: _False_
        '''
        return self._report[2]

    @deepdebug.setter
    def deepdebug(self, value):
        '''
        If deepdebug is set to _True_, it also sets _debug_ and _verbose_
        to _True_.

        @param: {Boolean}
        '''
        value = bool(value)
        if value:
            self._report[0:3] = [value]*3
        else:
            self._report[2]   = value

    @property
    def warnings(self):
        '''
        Prompts warning info in places where something is not completely wrong
        but there is no need to stop the execution.

        @return:  {Boolean}
        @default: _True_
        '''
        return self._report[3]

    @warnings.setter
    def warnings(self, value):
        '''
        @param: {Boolean}
        '''
        self._report[3] = bool(value)

    @property
    def errors(self):
        '''
        Prompts error info in places where something goes wrong.

        @return:  {Boolean}
        @default: _True_
        '''
        return self._report[4]

    @errors.setter
    def errors(self, value):
        '''
        @param: {Boolean}
        '''
        self._report[4] = bool(value)

    @property
    def overwrite(self):
        '''
        General overwrite setting for all the execution.
        It is OVER-RULED by local overwrite

        @return:  {Boolean}
        @default: _False_
        '''
        return self._ios[0]

    @overwrite.setter
    def overwrite(self, value):
        '''
        @param: {Boolean}
        '''
        self._ios[0] = bool(value)

    @property
    def stdout(self):
        '''
        STDOUT output status

        @return:  {Boolean}
        @default: _False_
        '''
        return self._ios[1]

    @stdout.setter
    def stdout(self, value):
        '''
        @param: {Boolean}
        '''
        self._ios[1] = bool(value)
        if value:
            self.log_file()

    ###########
    # METHODS #
    ###########
    def log_file(self, log_file = None):
        '''
        Manually select a file to print the output information.

        The resultant log-file is independent from the overwrite parameter.
        It opens as any regular python file (that is, it does overwrite).

        @param:    log_file
        @pdef:     Name of the log file
        @pdefault: _None_, defaults to STDERR
        @ptype:    {String}
        '''
        frmt = logging.Formatter(self._GENERAL_FORMAT.format(self._LOGNAME),
                                 self._TIME_FORMAT)
        if log_file is None:
            handler = logging.StreamHandler()
        else:
            handler = logging.FileHandler(log_file)
        handler.setFormatter(frmt)
        self._fd.addHandler(handler)
        if log_file is not None:
            self._fd.info('Logfile {0} created.'.format(log_file))

    def set_file(self, work_dir = os.getcwd()):
        '''
        Automatically create a file to print the output information.

        The log-file consists in the name of the current executing process,
        the process-identifier (pid) number and the .log extension.

        The resultant log-file is independent from the overwrite parameter.
        It opens as any regular python file (that is, it does overwrite).

        @param:    work_dir
        @pdef:     Name of the directory into which create the file.
        @pdefault: Current working directory.
        @ptype:    {String}
        '''
        script_name = os.path.split(os.path.splitext(sys.argv[0])[0])[1]
        log_file    = ".".join([script_name, str(os.getpid()), 'log'])
        self.log_file(os.path.join(work_dir, log_file))

    def decide_overwrite(self, local_overwrite):
        '''
        When local file overwrite has not been set (is None), the *Parameters*
        _overwrite_ attribute is called. Otherwise local overwrite
        specification has priority.

        @param: local_overwrite
        @pdef:  Selected status to control the overwrite of an already existing
                file.
        @ptype: {Boolean}

        @return: {Boolean}
        '''
        return self.overwrite if local_overwrite is None else local_overwrite

    def alert(self, level = 'verbose', message = None):
        '''
        Throws a message for the user.

        @param:    level
        @pdef:     specifies the minimum active level to actually show the
                   message.
        @pdefault: verbose
        @poptions: 'verbose', 'debug', 'deepdebug' or, equivalently,
                   0, 1, 2
        @ptype:    {String} or {Integer}

        param:     message
        @pdef:     specific message sent by the user.
        @pdefault: {None}
        @ptype:    {String}
        '''
        level, stat = self._active_level(level)
        if not stat:
            return

        callerID = inspect.getmodule(inspect.stack()[1][0]).__name__
        if callerID is '__main__':
            callerID = inspect.getmodule(inspect.stack()[1][0]).__file__
            callerID = os.path.splitext(callerID)[-1]
        callerID = '[' + callerID.upper() + '] '

        if message is None:
            message = ''

        if isinstance(message, list):
            for line in message:
                self.alert(level, line)
        else:
            if level == 'verbose':
                self._fd.info('{0}{1}'.format(callerID, message))
            else:
                self._fd.debug('{0}{1}'.format(callerID, message))

    def warn(self, message = None):
        '''
        Throw a warning message. Call when something should be said to the user
        about what's going on.

        @param:    message
        @pdef:     specific message sent by the user.
        @pdefault: {None}
        @ptype:    {String}

        '''
        if self.warnings is False:
            return

        callerID = inspect.getmodule(inspect.stack()[1][0]).__name__
        if callerID is '__main__':
            callerID = inspect.getmodule(inspect.stack()[1][0]).__file__
            callerID = os.path.splitext(callerID)[-1]
        callerID = '[' + callerID.upper() + '] '

        if isinstance(message, list):
            for line in message:
                self.warn(line)
        else:
            self._fd.warning('{0}{1}'.format(callerID, message))

    def throw(self, message = None, error = None, killit  = True):
        '''
        Throw an error. Call it when an error occur.
        If the _error_ attribute is _False_, it skips the effect, unless
        _killit_ is _True_, which will imply that we can not proceed given the
        error

        @param:    message
        @pdef:     specific message sent by the user.
        @pdefault: {None}
        @ptype:    {String}

        @param:    error
        @pdef:     {Exception} that has been captured with the error.
        @pdefault: {None}
        @ptype:    {Exception}

        @param:    killit
        @pdef:     order to stop the execution after the error has been
                   reported.
        @pdefault: _True_
        @ptype:    {Boolean}
        '''
        if self.errors is False and killit is False:
            return

        callerID = inspect.getmodule(inspect.stack()[1][0]).__name__
        if callerID is '__main__':
            callerID = inspect.getmodule(inspect.stack()[1][0]).__file__
            callerID = os.path.splitext(callerID)[-1]
        callerID = '[' + callerID.upper() + '] '

        self._fd.error('{0}{1}'.format(callerID, message))

        if error is not None:
            self._fd.error(error.message)
            self._fd.error(traceback.format_exc())

        if killit:
            sys.exit(-9)

    def success(self):
        '''
        Tells that the program has successfully finished
        '''
        self._fd.info('[SUCCESS!!]: -- Program ended as expected.')
        sys.exit(0)

    def countdown(self, max_time):
        '''
        Prints a countdown in place.
        Put it in a loop if you are waiting for something.

        @param:    max_time
        @pdef:     time to wait. in seconds.
        @ptype:    {integer}
        '''
        t  = str(datetime.timedelta(seconds=max_time))
        n  = time.localtime()
        s1 = 'Waiting for: {0} hours'.format(t)
        s2 = 'Wait started at {0}'.format(time.strftime('%X', n))
        s3 = 'on {0}'.format(time.strftime('%Y-%m-%d', n))
        sys.stderr.write('{0}\t{1} {2}\n\n'.format(s1, s2, s3))
        while max_time > 0:
            t = str(datetime.timedelta(seconds=max_time))
            sys.stderr.write('Remaining: {0} hours'.format(t))
            time.sleep(1)
            max_time -= 1
            if bool(max_time):
                sys.stderr.write('\r')
            else:
                sys.stderr.write('\r')
                t = str(datetime.timedelta(seconds=max_time))
                sys.stderr.write('Remaining: {0} hours'.format(t))
                sys.stderr.write('\n')

    ###################
    # PRIVATE METHODS #
    ###################
    def _active_level(self, level):
        '''
        Decide if a given level is active or not.

        @param:    level
        @pdef:     level to check if it is active.
        @pdefault: {None}
        @ptype:    {String}

        @return: {Boolean}
        '''
        try:
            level = int(level)
            stat  = ((level == 0 and self.verbose) or
                    (level  == 1 and self.debug)   or
                    (level  == 2 and self.deepdebug))
            return ['verbose', 'debug', 'deepdebug'][level], stat
        except Exception:
            stat = ((level == 'verbose'   and self.verbose) or
                   (level  == 'debug'     and self.debug)   or
                   (level  == 'deepdebug' and self.deepdebug))
            return level, stat



from SBI.beans.StorableObject import StorableObject
from SBI.beans.File           import File
from .CDhit         import CDhit
from .CDhitHomolog  import CDhitHomolog

class CDhitList(StorableObject):
    def __init__(self, cdhitfile):
        self._clusters  = []
        self._allseqids = {}
        self._file      = File(file_name = cdhitfile)

        self._parse_file()

    @property
    def clusters(self): return self._clusters

    def get_cluster4sequence(self, sequence):
        if sequence in self._allseqids:
            return self._clusters[self._allseqids[sequence]]
        else:
            return None

    def is_in_cluster(self, sequence):
        c = self.get_cluster4sequence(sequence)
        if c is None: return 'N'
        else:         return 'M' if c.is_master(sequence) else 'H'

    def add_cluster(self, cluster):
        self._clusters.append(cluster)

    def add_sequence2cluster(self, sequence, clusterid = None):
        if clusterid is None:
            self.clusters[-1].add_sequence(sequence)
            self._allseqids[sequence.name] = len(self.clusters) - 1
        else:
            for x in range(len(self._clusters)):
                if self._clusters[x].identifier == clusterid:
                    self._clusters[x].add_sequence(sequence)
                    self._allseqids[sequence.name] = x
                    break

    def dictionary_role_summary(self):
        data = {'master':[], 'homolog':[]}
        for c in self.clusters:
            data['master'].append(c.master.name)
            for s in c.sequences:
                data['homolog'].append(s)
        return data

    def _parse_file(self):
        for line in self._file.descriptor:
            if line.startswith('>'):
                c = CDhit(clusterid = line.split()[-1].strip())
                self.add_cluster(c)
            else:
                data = line.split()[1:]
                h    = CDhitHomolog(name = data[1], length = data[0], homology = data[-1])
                self.add_sequence2cluster(sequence = h)
        self._file.close()

    def __repr__(self):
        text = []
        for c in self.clusters:
            text.append('{0}'.format(c))
        return '\n'.join(text)



def _sequencer(pdb, mode):
    seq = ''
    idx = []
    for aa in range(len(pdb.aminoacids)):
        if aa == 0:
            if mode == 'seq':
                seq += pdb.aminoacids[aa].single_letter
            if mode == 'str':
                seq += pdb.aminoacids[aa].secondary_structure
            if mode == 'idx':
                idx.append(pdb.aminoacids[aa].identifier)
        else:
            if pdb.aminoacids[aa].follows(pdb.aminoacids[aa - 1]):
                if mode == 'seq':
                    seq += pdb.aminoacids[aa].single_letter
                if mode == 'str':
                    seq += pdb.aminoacids[aa].secondary_structure
                if mode == 'idx':
                    idx.append(pdb.aminoacids[aa].identifier)
            else:
                id_distance = -1 * \
                    pdb.aminoacids[aa].identifier_distance(
                        pdb.aminoacids[aa - 1])
                if id_distance > 1:
                    for x in range(id_distance - 1):
                        if mode == 'idx':
                            idx.append('X')
                        else:
                            seq += 'x'
                else:
                    if mode == 'idx':
                        idx.append('X')
                    else:
                        seq += 'x'
                if mode == 'seq':
                    seq += pdb.aminoacids[aa].single_letter
                if mode == 'str':
                    seq += pdb.aminoacids[aa].secondary_structure
                if mode == 'idx':
                    idx.append(pdb.aminoacids[aa].identifier)

    if mode == 'idx':
        return ";".join(idx)
    else:
        return seq



"""
Executable

author: jbonet
date:   03/2013

@oliva's lab
"""

import os
import subprocess
from SBI import SBIglobals


class Executable(object):
    """
    Checks the integrity of an executable program
    """
    def __init__(self, executable, path=None, variable_path=None):
        """
        @type  executable: String
        @param executable: Name of the executable file

        @type  path: String
        @param path: Putative path to the executable (if case is not in $PATH)

        @type  variable_path: String
        @param variable_path: Name of the enviroment variable with the path
        """

        self._exec = executable
        self._path = path

        if self._exec is None:
            raise AttributeError("The executable name MUST be specified")

        if variable_path is None:
            if path is not None:
                self._path = os.path.abspath(path)
            else:
                if not self._load_executable_path():
                    raise EnvironmentError("")
        else:
            self._load_variable_path(variable_path=variable_path)

        self._check_executable()

        self._command = []
        self._command.append(self.full_executable)

        self._outfile = None
        self._stdout  = None

    #
    # ATTRIBUTES
    #
    @property
    def executable(self):
        return self._exec

    @property
    def path(self):
        return self._path

    @property
    def command(self):
        return self._command

    @property
    def outfile(self):
        return self._outfile

    @outfile.setter
    def outfile(self, value):
        self._outfile = value

    @property
    def full_executable(self):
        return os.path.join(self._path, self._exec)

    #
    # METHODS
    #
    def add_attribute(self, attribute_value, attribute_id=None):
        if attribute_id is not None:
            self._command.append(attribute_id)
        self._command.append(str(attribute_value))

    def add_parameter(self, parameter):
        self._command.append(str(parameter))

    def clean_command(self):
        self._command = []
        self._command.append(self.full_executable)

    def execute(self, stdout=False, silent=False):
        """
        Executes the commands
        @type  stdout: Boolean
        @param stdout: determines if the output is through stdout
        """
        if self.command is None:
            return False

        if not stdout:
            stdoutPIPE = open('/dev/null', 'w')
        else:
            stdoutPIPE = subprocess.PIPE

        if silent:
            stderrPIPE = open('/dev/null', 'w')
        else:
            stderrPIPE = subprocess.PIPE

        SBIglobals.alert('debug', self, '\tExecuting command:\n\t{0}\n'.format(" ".join(self.command)))

        p = subprocess.Popen(self.command, stdout=stdoutPIPE,
                             stderr=stderrPIPE)

        out, err  = p.communicate()

        if stdout:
            self._stdout = out

        if not silent and err.strip() != '':
            raise SystemError(err)

    #
    # PRIVATE METHODS
    #
    def _load_variable_path(self, variable_path):
        """
        Retrieves the path from a variable_path
        """
        try:
            self._path = os.environ[variable_path]
        except KeyError:
            raise EnvironmentError("The given Environment Variable %s is not defined" %variable_path)

    def _load_executable_path(self):
        """
        Retrieves the executable path in case self._path is None
        """
        if self._path is not None:
            return

        search = ["which", self.executable]
        p = subprocess.Popen(search, stdout=subprocess.PIPE,
                             stderr=subprocess.PIPE)
        out, err = p.communicate()

        if out != '':
            self._path = os.path.split(out.strip())[0]
            return True
        else:
            return False

    def _check_executable(self):
        """
        Checks that the final executable can be executed
        """
        if not os.path.isfile(self.full_executable):
            raise SystemError("The given executable file %s does not exist" %self.full_executable)
        if not os.access(self.full_executable, os.X_OK):
            raise SystemError("The given executable %s can not be executed" %self.full_executable)

    '''OVERLOAD DEFAULT FUNCTIONS'''
    def __repr__(self):
        return " ".join(self._command)

    def __str__(self):
        return repr(self)



import os, sys, subprocess
import BioLib.Structure.PDB as PDB

class ZDock(object):
    '''
    Class to manage a ZDock transform file
    '''
    def __init__(self, ZDock_file):
        '''
        Call the parsing method
        '''
        self.decoys = []
        self.__parse(ZDock_file)

    def __parse(self, ZDock_file):
        '''
        '''
        ZDock_fo = open(ZDock_file, 'r')
        self.n, self.spacing, self.switch_num = ZDock_fo.readline().strip('\n').split('\t')
        self.rec_rand1 = 0.0
        self.rec_rand2 = 0.0
        self.rec_rand3 = 0.0
        if self.switch_num != '':
            self.rec_rand1, self.rec_rand2, self.rec_rand3 = ZDock_fo.readline().strip('\n').split('\t')
        self.lig_rand1, self.lig_rand2, self.lig_rand3 = ZDock_fo.readline().strip('\n').split('\t')
        self.rec, self.r1, self.r2, self.r3 = ZDock_fo.readline().strip('\n').split('\t')
        self.lig, self.l1, self.l2, self.l3 = ZDock_fo.readline().strip('\n').split('\t')
        if self.switch_num == '1':
            temp_name = self.rec;
            self.rec = self.lig;
            self.lig = temp_name;
        num = 1
        for line in ZDock_fo:
            decoy = line.strip('\n').split('\t')
            self.decoys.append(ZDecoy(num, self, float(decoy[0]), float(decoy[1]), float(decoy[2]), int(decoy[3]), int(decoy[4]), int(decoy[5]), float(decoy[6])))
            num += 1
        ZDock_fo.close()

    # GETTERS #

    def get_decoys(self):
        '''
        Return all decoys
        '''
        return self.decoys

    def get_decoy(self, num):
        '''
        Return a decoy by its number
        '''
        for decoy in self:
            if decoy.get_num() == num:
                return decoy
        return None

    # METHODS #

    def get_num_decoys(self):
        return len(self.decoys)

    def print_structures(self, static_structure, mobile_structure, pdb_file, single_pdb=False):
        '''
        Print all PDB decoys
        '''
        decoy_list = []
        for decoy in self:
            if singlePDB:
                decoy_structure = decoy.get_structure(mobile_structure)
                decoy_list.append((static_structure, decoy_structure))
            else:
                decoy.print_structure(static_structure, decoy_structure, pdb_file+'_'+str(decoy.get_num())+'.pdb')
        if singlePDB:
            PDB.write_pdb(decoy_list, pdb_file+'.pdb', multi_chain=True, multi_model=True)

    # MAGIC METHODS #

    def __iter__(self):
        '''
        Iterates over each decoy
        '''
        for decoy in self.decoys:
            yield decoy

    def __str__(self):
        '''
        toString method
        '''
        return '\n'.join([decoy.__str__() for decoy in self.decoys])

class ZDecoy(object):
    '''
    Represents a decoy result from a ZDock transform file
    '''
    def __init__(self, num, zdock, alpha, beta, gamma, x, y, z, energy):
        '''
        '''
        self.num = num
        self.zdock = zdock
        self.alpha = alpha
        self.beta = beta
        self.gamma = gamma
        self.x = x
        self.y = y
        self.z = z
        self.energy = energy

    # GETTERS #

    def get_num(self):
        return self.num

    def get_energy(self):
        return self.energy

    def get_tranlation_vector(self):
        return (self.x, self.y, self.z)
   
    def get_rotation_vector(self):
        return (self.alpha, self.beta, self.gamma)

    def get_structure(self, mobile_structure):
        '''
        Gets the structure of the given docking result
        '''
        ZDock_create = os.path.abspath(os.path.join(os.path.dirname(__file__), 'ZDock_create'))
        if self.zdock.switch_num != '':
            create_cmd = '%s %s %s %s %s %s %s %s %s %s %s %s %s %s %s %s %s %s %s %s %s %s' % (
                          ZDock_create, self.zdock.switch_num,
                          self.zdock.rec_rand1, self.zdock.rec_rand2, self.zdock.rec_rand3,
                          self.zdock.lig_rand1, self.zdock.lig_rand2, self.zdock.lig_rand3, 
                          self.zdock.r1, self.zdock.r2, self.zdock.r3,
                          self.zdock.l1, self.zdock.l2, self.zdock.l3,
                          self.alpha, self.beta, self.gamma,
                          self.x, self.y, self.z,
                          self.zdock.n, self.zdock.spacing)
        else:
            create_cmd = '%s %s %s %s %s %s %s %s %s %s %s %s %s %s %s %s %s %s' % (
                          ZDock_create, 
                          self.zdock.lig_rand1, self.zdock.lig_rand2, self.zdock.lig_rand3, 
                          self.zdock.r1, self.zdock.r2, self.zdock.r3,
                          self.zdock.l1, self.zdock.l2, self.zdock.l3,
                          self.alpha, self.beta, self.gamma,
                          self.x, self.y, self.z,
                          self.zdock.n, self.zdock.spacing)
        
        p = subprocess.Popen(create_cmd, stdin=subprocess.PIPE, stdout=subprocess.PIPE, stderr=subprocess.PIPE, shell=True)
        create_out, create_err = p.communicate(str(mobile_structure))

        structure = PDB.read_pdb(create_out, from_string=True)
        return structure

    def print_structure(self, static_structure, mobile_structure, pdb_file):
        decoy_structure = self.get_structure(mobile_structure)
        PDB.write_pdb((static_structure, decoy_structure), pdb_file+'.pdb', multi_chain=True, multi_model=False)

    def __str__(self):
        return '%d %f %d %d %d %f %f %f' % (self.num, self.energy, self.x, self.y, self.z, self.alpha, self.beta, self.gamma)
